/*
 *
 *
 * Updated: 28th February 2003

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <crtcore.h>
#include <exception>
#include <string.h>
#include <limits.h>
#include <stdlib.h>

namespace std
{
	namespace
	{
		const uintptr_t BADALLOC_FLAG = uintptr_t(1) << (sizeof(uintptr_t) * CHAR_BIT - 1);
		const char		s_DefaultMessage[]			=	"std::exception";
		const char		s_DefaultBadAllocMessage[]	=	"std::bad_alloc";

		inline
		bool IsDefaultMessage(char const *s)
		{
			return s == s_DefaultMessage || s == s_DefaultBadAllocMessage;
		}

		inline
		char const *AddBadAllocFlag(char const *s)
		{
			return (char const *)(BADALLOC_FLAG | (uintptr_t)s);
		}

		inline
		char const *StripBadAllocFlag(char const *s)
		{
			return (char const *)(~(BADALLOC_FLAG) & uintptr_t(s));
		}

		inline
		bool IsBadAllocString(char const *s)
		{
			return	s == s_DefaultBadAllocMessage ||
					StripBadAllocFlag(s) == s;
		}
	}

	exception::exception() __CRUNTINY_THROW0()
		: m_What(s_DefaultMessage)
	{}
	
	exception::exception(char const * const & message) __CRUNTINY_THROW0()
		: m_What(_strdup(message))
	{
		if (m_What == 0)
		{
			m_What = s_DefaultMessage;
		}
	}

	exception::exception(char const * const &message, bad_alloc_marker *) __CRUNTINY_THROW0()
		: m_What(_strdup(message))
	{
		if (m_What != 0)
		{
			m_What = AddBadAllocFlag(m_What);
		}
		else
		{
			m_What = s_DefaultBadAllocMessage;
		}
	}

	exception::exception(exception const & src) __CRUNTINY_THROW0()
		: m_What(0)
	{
		if(IsDefaultMessage(src.m_What))
		{
			m_What = src.m_What;
		}
		else
		{
			m_What = _strdup(src.m_What);

			if (m_What == 0)
			{
				if(IsBadAllocString(src.m_What))
				{
					m_What = s_DefaultBadAllocMessage;
				}
				else
				{
					m_What = s_DefaultMessage;
				}
			}
		}
	}

	exception & exception::operator =(exception const & rhs) __CRUNTINY_THROW0()
	{
		if (!IsDefaultMessage(m_What))
		{
			free(const_cast<char*>(StripBadAllocFlag(m_What)));
		}

		if (IsDefaultMessage(rhs.m_What))
		{
			m_What = rhs.m_What;
		}
		else
		{
			m_What = _strdup(rhs.m_What);

			if (m_What == 0)
			{
				if(IsBadAllocString(rhs.m_What))
				{
					m_What = s_DefaultBadAllocMessage;
				}
				else
				{
					m_What = s_DefaultMessage;
				}
			}
		}

		return *this;
	}

	exception::~exception() __CRUNTINY_THROW0()
	{
		if (!IsDefaultMessage(m_What))
		{
			free(const_cast<char*>(StripBadAllocFlag(m_What)));
		}
	}

	char const * exception::what() const
	{
		return StripBadAllocFlag(m_What);
	}

} // namespace std
