/*

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <crtcore.h>
#include "compiler.h"
#include "module_data.h"
#include "application.h"
#include "process_data.h"
#include "thread_data.h"

extern "C"
BOOL WINAPI DllMain(HINSTANCE hInstance,DWORD dwReason,LPVOID lpv);

extern "C"
BOOL WINAPI _DllMainCRTStartup(HINSTANCE hInstance,
							      DWORD dwReason,
								  LPVOID lpv)
{
	if (dwReason == DLL_PROCESS_ATTACH)
	{
		__Compiler::ProcessData::Init();
		__Compiler::ModuleData::Init();
		__Compiler::InitUserData();
	}
#ifdef _MT
	if (dwReason == DLL_THREAD_ATTACH)
	{
		//ensure we have a thread data
		/*__Compiler::ThreadData * ptdata =*/ __Compiler::ThreadData::GetInstance();

	}
#endif
	
	const BOOL res = DllMain(hInstance, dwReason, lpv);

#ifdef _MT
	if (dwReason == DLL_THREAD_DETACH)
	{
		//Here is the tricky part
		//if thread data exist we destroy it here
		//however other dlls may re-create it in their DllMains
		//This is not a problem since they will also
		//destroy it here
		__Compiler::ThreadData * ptdata = __Compiler::ThreadData::GetInstance();
		if (ptdata)
		{
			ptdata->DestroyInstance();
		}
	}
#endif


	if ((!res && dwReason == DLL_PROCESS_ATTACH) ||
		(dwReason == DLL_PROCESS_DETACH))
	{
		__Compiler::ModuleData::Term();
		__Compiler::ProcessData::Term();
	}
	
	return res;
}

