/*
 *
 *
 * Updated: 28th February 2003

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __COMPILER_EH_CONTEXT_H
#define __COMPILER_EH_CONTEXT_H

#include "exception_handling.h"

namespace __Compiler
{
	namespace EH
	{
		class GuardDescriptor;
		
		class ExceptionContext
		{
		friend EXCEPTION_DISPOSITION CallHandler(HandlerPtr pfn,
												 ExceptionContext & context,
												 const GuardDescriptor & guard,
												 BasicRegistrationNode * pMarkerRN, 
												 bool recursive);
		public:
			ExceptionContext():
				m_pRecord(0),
				m_pContext(0),
				m_pDispatcherContext(0)
			{
			}
			ExceptionContext(ExceptionRecord * pExcept, Context * pContext, void * pDC):
				m_pRecord(pExcept),
				m_pContext(pContext),
				m_pDispatcherContext(pDC)
			{
			}

			bool IsValid() const
				{ return m_pRecord != 0; }

			bool IsCppException() const
				{ return m_pRecord->IsCpp(); }

			const ThrownObject & ExceptionObject() const
				{ return m_pRecord->Object(); }
			ThrownObject & ExceptionObject()
				{ return m_pRecord->Object(); }
			
			DWORD ExceptionCode() const
				{ return m_pRecord->ExceptionCode; }

			DWORD ExceptionFlags() const
				{ return m_pRecord->ExceptionFlags; }

			HandlerPtr CustomHandler() const
			{
				if (m_pRecord->IsCppCompatible())
					return m_pRecord->Object().CustomHandler();
				return 0;
			}

			__CRUNTINY_DECLSPEC_NORETURN
			void Rethrow() const
				{ m_pRecord->Throw(); }


			void CopyExceptionObject(RegistrationNode * pRN, 
									 const HandlerType * pCatch, 
									 const CatchableType * pConv)
				{ m_pRecord->CopyObject(pRN, pCatch, pConv); }

			EXCEPTION_DISPOSITION CallHandler(HandlerPtr pfn,
											  const GuardDescriptor & guard,
											  BasicRegistrationNode * pMarkerRN, 
											  unsigned char recursive);
			void UnwindNestedFrames(__Platform::ExceptionFrame * pFrame)
				{ pFrame->UnwindNestedFrames(m_pRecord); }
		private:
			ExceptionRecord * m_pRecord;
			Context * m_pContext;
			void * m_pDispatcherContext;
		};
	}

} // namespace __Compiler


#endif /* __COMPILER_EH_CONTEXT_H */
