/*

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <crtcore.h>
#include "compiler.h"
#include "eh_guard_descriptor.h"
#include "thread_data.h"
#include "call.h"
#include <new>


namespace __Compiler
{
	namespace EH
	{
		EXCEPTION_DISPOSITION CxxFrameHandler(ExceptionContext & context, 
											  GuardDescriptor & guard,
											  BasicRegistrationNode * pMarkerRN, 
											  bool recursive);

		GuardDescriptor::FrameInfo::FrameInfo(const ThrownObject & object):
			m_pThrownObject(&object)
		{
			FrameInfo *& pChain = ThreadData::GetInstance()->FrameInfoChain();
			m_pNext = pChain;
			pChain = this;
		}

		GuardDescriptor::FrameInfo::~FrameInfo()
		{
			FrameInfo *& pChain = ThreadData::GetInstance()->FrameInfoChain();
			if (pChain == this)
			{
				pChain = m_pNext;
				return;
			}
			
			FrameInfo * pInfo = 0;
			for(pInfo = pChain; pInfo->m_pNext != this; pInfo = pInfo->m_pNext)
			{
				if (pInfo->m_pNext == 0)
					inconsistency();
			}
			pInfo->m_pNext = m_pNext;
		}


		
		const TryBlockMapEntry * GuardDescriptor::GetRangeOfTrysToCheck(unsigned int * pStart, 
																		unsigned int * pEnd) const
		{
			const int curState = m_pRN->state;
			int CatchDepth = m_CatchDepth;
			unsigned start = m_pFuncInfo->nTryBlocks;
			unsigned temp = start;                                             
			unsigned end = start; 
			while (CatchDepth >= 0)
			{
				if (start-- == -1)
					inconsistency();

				const TryBlockMapEntry * const pEntry = m_pFuncInfo->pTryBlockMap + start;
				if ( (pEntry->tryHigh >= curState ||
					  curState > pEntry->catchHigh) &&
					 start == -1 )
				{
					end = temp;
					temp = start;
					--CatchDepth;
				}
			}
			*pStart = ++start;                                          
			*pEnd = end;
			if (end > m_pFuncInfo->nTryBlocks || start >= end)
				inconsistency();
			return m_pFuncInfo->pTryBlockMap + start;
		}

		int GuardDescriptor::FrameUnwindFilter(EXCEPTION_POINTERS * pExPtrs)
		{
			if (pExPtrs->ExceptionRecord->ExceptionCode == CPP_EXCEPTION)
			{
				ThreadData::GetInstance()->ProcessingThrow() = 0;
				std::terminate();
			}
			return EXCEPTION_CONTINUE_SEARCH;
		}

		void GuardDescriptor::FrameUnwindToState(int targetState)
		{
			int curState = m_pRN->state;
			int & ProcessingThrow = ThreadData::GetInstance()->ProcessingThrow();
			++ProcessingThrow;
			__try 
			{
				while (curState != targetState)
				{
					if (curState <= -1 || curState >= m_pFuncInfo->maxState)
						inconsistency();
					int nxtState = m_pFuncInfo->pUnwindMap[curState].toState;
					__try
					{
						if (m_pFuncInfo->pUnwindMap[curState].action != 0)
						{
							m_pRN->state = nxtState;
							CallOnFrame0(m_pFuncInfo->pUnwindMap[curState].action,
										 m_pRN->GetStackFrame());
						}
					}
					__except(FrameUnwindFilter((EXCEPTION_POINTERS*)_exception_info()))
					{
						
					}
					curState = nxtState;
				}
			}
			__finally
			{
				if (ProcessingThrow > 0)
						--ProcessingThrow;
			}
			
			if (curState != targetState)
				inconsistency();
			m_pRN->state = curState;
		}

		EXCEPTION_DISPOSITION GuardDescriptor::CatchGuardHandler(ExceptionRecord * pExcept, 
																 CatchGuardRN * pRN, 
																 Context * pContext, 
																 void * /*pDC*/)
		{
			ExceptionContext context(pExcept, pContext, 0);
			return CxxFrameHandler(context, 
								   pRN->StoredValue(),
								   pRN, 
								   false);
			
		}


		void * GuardDescriptor::CallCatchBlock2(void * handlerAddress)
		{
			CatchGuardRN CGRN(m_pRN, m_pFuncInfo, CatchGuardHandler, m_CatchDepth + 1);
			CGRN.SetCurrent();

			void * continuationAddress = CallOnFrame0(handlerAddress, m_pRN->GetStackFrame());
			CGRN.Remove();
			return continuationAddress;
		}

		int GuardDescriptor::ExFilterRethrow(EXCEPTION_POINTERS * pExPtrs)
		{
			const ExceptionRecord * pExcept = (const ExceptionRecord*)(pExPtrs->ExceptionRecord);
			if (!pExcept->IsCpp() || pExcept->Object().IsValid() != 0)
				return EXCEPTION_CONTINUE_SEARCH;
			return EXCEPTION_EXECUTE_HANDLER;
		}

		void * GuardDescriptor::CallCatchBlock(ExceptionContext & context, void * handlerAddress)
		{
			ThreadData * const pThreadData = ThreadData::GetInstance();
			void * continuationAddress = handlerAddress;
			bool ExceptionObjectDestroyed = false;
			
			void * const saveESP = m_pRN->GetStack();
			const ExceptionContext SavedContext = pThreadData->CurrentException();                                         

			DWORD frame_info[sizeof(FrameInfo)/sizeof(DWORD)];
			FrameInfo * pFrameInfo = new (&frame_info) FrameInfo(context.ExceptionObject());
			pThreadData->CurrentException() = context;

			__try
			{
				__try
				{
					continuationAddress = CallCatchBlock2(handlerAddress);
				}
				__except(ExFilterRethrow((EXCEPTION_POINTERS*)_exception_info()))
				{
					const UnwindMapEntry * pUnwindMap = m_pFuncInfo->pUnwindMap;
					int cState = m_pRN->state;                                                                      
					const TryBlockMapEntry * pTryBlockMap = m_pFuncInfo->pTryBlockMap;
					for(unsigned i = 0; i < m_pFuncInfo->nTryBlocks; ++i)
					{
						if (cState <= pTryBlockMap[i].tryHigh ||
							cState > pTryBlockMap[i].catchHigh)
							continue;

						cState = pTryBlockMap[i].tryHigh + 1;                                                         
						cState = pUnwindMap[cState].toState;
					}
					FrameUnwindToState(cState);                                                
					continuationAddress = 0;
				}
			}
			__finally
			{
				m_pRN->SetStack(saveESP);
				pFrameInfo->~FrameInfo();
				pThreadData->CurrentException() = SavedContext;
				if (context.IsCppException() &&
					!ExceptionObjectDestroyed &&
					continuationAddress != 0 &&
					IsToBeDestroyed(context.ExceptionObject()))
					context.ExceptionObject().Destruct(_abnormal_termination() != 0);
			}
			return continuationAddress;
		}
 	
		void GuardDescriptor::Catch(ExceptionContext & context,
								    const HandlerType * pCatch, 
								    const CatchableType * pConv, 
								    const TryBlockMapEntry * pEntry, 
								    BasicRegistrationNode * pMarkerRN)
		{
			if (pConv != 0)
				context.CopyExceptionObject(m_pRN, pCatch, pConv); 
			
			context.UnwindNestedFrames(pMarkerRN != 0 ? pMarkerRN : m_pRN);
			
			FrameUnwindToState(pEntry->tryLow);

			m_pRN->state = pEntry->tryHigh + 1;

			void * const continuationAddress = CallCatchBlock(context, pCatch->addressOfHandler);
			if (continuationAddress)
				JumpToContinuation(continuationAddress); 
		}

		void __stdcall GuardDescriptor::JumpToContinuation(void * target)
		{
			RegistrationNode::GetCurrent()->Remove();
			void * const targetEBP = m_pRN->GetStackFrame();
			void * const targetStack = m_pRN->GetStack();
			JumpToFunction(target, targetStack, targetEBP);
		}

		bool GuardDescriptor::IsToBeDestroyed(const ThrownObject & object) const
		{
			for(const FrameInfo * pInfo = ThreadData::GetInstance()->FrameInfoChain();
				pInfo != 0;
				pInfo = pInfo->Next())
			{
				if(pInfo->Object() == object)
					return false;
			}
			return true;
		}		

	}
}





