/*
 *
 *
 * Updated: 28th February 2003

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __COMPILER_EH_GUARD_DESCRIPTOR_H
#define __COMPILER_EH_GUARD_DESCRIPTOR_H

#include "exception_handling.h"

namespace __Compiler
{
	namespace EH
	{
		class ExceptionContext;

		class GuardDescriptor
		{
		friend EXCEPTION_DISPOSITION CallHandler(HandlerPtr pfn,
												 ExceptionContext & context,
												 const GuardDescriptor & guard,
												 BasicRegistrationNode * pMarkerRN, 
												 bool recursive);
		public:
			class FrameInfo;

			GuardDescriptor(RegistrationNode * pRN,
							const FuncInfo * pFuncInfo,
							int CatchDepth):
				m_pFuncInfo(pFuncInfo),
				m_pRN(pRN),
				m_CatchDepth(CatchDepth)
			{
			}

			bool IsValid() const
				{ return ( (m_pFuncInfo->magicNumber & MS_MAGIC_MASK) == MS_MAGIC ); }
			bool MayBeUnwound() const
				{ return m_pFuncInfo->pUnwindMap != 0; }

			int CurrentState() const
				{ return m_pRN->state; }
			int MaxState() const
				{ return m_pFuncInfo->maxState; }
			int CatchDepth() const
				{ return m_CatchDepth; }

			const TryBlockMapEntry * GetRangeOfTrysToCheck(unsigned int * pStart, 
														   unsigned int * pEnd) const;
			void FrameUnwindToState(int targetState);
			void Catch(ExceptionContext & context,
					   const HandlerType * pCatch, 
					   const CatchableType * pConv, 
					   const TryBlockMapEntry * pEntry, 
					   BasicRegistrationNode * pMarkerRN);
		private:
			class CatchGuardRN;

			void * CallCatchBlock(ExceptionContext & context, void * handlerAddress);
			void * CallCatchBlock2(void * handlerAddress);
			void __stdcall JumpToContinuation(void * target);
			bool IsToBeDestroyed(const ThrownObject & object) const;

			static int FrameUnwindFilter(EXCEPTION_POINTERS * pExPtrs);
			static int ExFilterRethrow(EXCEPTION_POINTERS * pExPtrs);

			static EXCEPTION_DISPOSITION CatchGuardHandler(ExceptionRecord * pExcept, 
														   CatchGuardRN * pRN, 
														   Context * pContext, 
														   void * /*pDC*/);

			const FuncInfo * m_pFuncInfo;
			RegistrationNode * m_pRN;
			int m_CatchDepth;
		};

		class GuardDescriptor::CatchGuardRN : public BasicRegistrationNode
		{
		public:
			CatchGuardRN(RegistrationNode * pRN, 
						const FuncInfo * pFuncInfo,
						void * Handler,
						int CatchDepth):
				BasicRegistrationNode(Handler),
				m_Descriptor(pRN,pFuncInfo,CatchDepth)
			{
			}

			GuardDescriptor & StoredValue()
				{ return m_Descriptor; }
		private:
			GuardDescriptor m_Descriptor; 

			//not to be implemented
			CatchGuardRN(const CatchGuardRN &);
			CatchGuardRN & operator=(const CatchGuardRN &);
		};

		class GuardDescriptor::FrameInfo
		{
		public:
			FrameInfo(const ThrownObject & object);
			~FrameInfo();

			const ThrownObject & Object() const
				{ return *m_pThrownObject; }
			const FrameInfo * Next() const
				{ return m_pNext; }
		private:
			const ThrownObject * m_pThrownObject;
			FrameInfo * m_pNext;

			FrameInfo(const FrameInfo &);
			FrameInfo & operator=(const FrameInfo &);
		};

	}

} // namespace __Compiler


#endif /* __COMPILER_EH_GUARD_DESCRIPTOR_H */
