/*
 *
 *
 * Updated: 28th February 2003

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __COMPILER_EXCEPTION_HANDLING_H
#define __COMPILER_EXCEPTION_HANDLING_H

#include "compiler.h"
#include "seh.h"
#include "rtti.h"

namespace __Compiler
{
	namespace EH
	{
		extern const DWORD CPP_EXCEPTION;
		extern const DWORD MS_MAGIC;
		extern const DWORD MS_MAGIC_MASK;
		
		struct FuncInfo;
		struct HandlerType;

		class ExceptionRecord;
		class RegistrationNode;
		class BasicRegistrationNode;
		

		typedef SEH::Context Context;
		typedef ::_s__CatchableType CatchableType;
		typedef ::_s__CatchableTypeArray CatchableTypeArray;
		typedef ::_s__ThrowInfo ThrowInfo;

		typedef EXCEPTION_DISPOSITION (__cdecl * HandlerPtr)(
							ExceptionRecord * pExcept,
							RegistrationNode * pRN,
							Context * pContext,
							void * pDC,
							const FuncInfo * pFuncInfo,
							int CatchDepth,
							BasicRegistrationNode * pMarkerRN,
							BOOL recursive);
#pragma pack(push)
#pragma pack(8)

		/*  these types are known to compiler without declaration
		the declaration is here for convenience


		struct _s__CatchableType
		{
			unsigned properties;
			_TypeDescriptor * pType; 
			_PMD thisDisplacement;
			int sizeOrOffset;
			void (*)() copyFunction; 
		};

		struct _s__CatchableTypeArray
		{
			int nCatchableTypes;
			const _s__CatchableType (* arrayOfCatchableTypes)[];
		};

		struct _s__ThrowInfo
		{
			unsigned attributes;
			void * pmfnUnwind;
			HandlerPtr pForwardCompat;
			const _s__CatchableTypeArray * pCatchableTypeArray;
		};

		*/

		struct UnwindMapEntry
		{
			int toState;
			void (*action)(void);
		};

		//describes a catch() block handler
		struct HandlerType
		{
			unsigned int adjectives;
			RTTI::TypeDescriptor * pType;
			int dispCatchObj;
			void * addressOfHandler;
		};

		struct TryBlockMapEntry
		{
			int tryLow;
			int tryHigh;
			int catchHigh;
			int nCatches;
			const HandlerType * pHandlerArray;
		};

		struct FuncInfo
		{
			unsigned int magicNumber;
			int maxState;
			const UnwindMapEntry * pUnwindMap;
			unsigned int nTryBlocks;
			const TryBlockMapEntry * pTryBlockMap;
			unsigned int nIPMapEntries;
			void * pIPtoStateMap;
		};

		class ThrownObject
		{
		public:
			ThrownObject(void * pObject, const ThrowInfo * pObjectInfo):
				m_pObject(pObject),
				m_pObjectInfo(pObjectInfo)
			{
			}

			bool IsValid() const
				{ return m_pObject != 0; }

			HandlerPtr CustomHandler() const
				{ return (HandlerPtr)(m_pObjectInfo->pForwardCompat); }

			bool operator==(const ThrownObject & rhs) const
				{ return m_pObject == rhs.m_pObject; }

			const CatchableType * FindMatch(const HandlerType * pCatch);
			void Destruct(bool fThrowNotAllowed);
			void Copy(void ** pCatchBuffer, const HandlerType * pCatch, const CatchableType * pConv);
		private:
			static bool TypeMatch(const HandlerType * pCatch, 
								  const CatchableType * pCatchable, 
								  const ThrowInfo * pThrow);

			void * m_pObject;
			const ThrowInfo * m_pObjectInfo;
		};
		
		class ExceptionRecord : public SEH::ExceptionRecord
		{
		// Construction
		public:
			ExceptionRecord(ThrownObject & object, 
							void * pExceptionAddress);
		// Accessors
		public:
			const ThrownObject & Object() const
				{ return *(ThrownObject*)&ExceptionInformation[1]; }

			ThrownObject & Object()
				{ return *(ThrownObject*)&ExceptionInformation[1]; }
			
			bool IsCpp() const
				{ return (IsCppCompatible() && NumberParameters == 3); }

			bool IsCppCompatible() const
				{ return (ExceptionCode == CPP_EXCEPTION && Magic() == MS_MAGIC);  }

			void Throw() const
				{ __Platform::RaiseException(this); }

			void CopyObject(RegistrationNode * pRN, 
							const HandlerType * pCatch, 
							const CatchableType * pConv);

			/*inline IsSelfHandled() const
			{
				return (pObjectInfo() != 0 && pObjectInfo()->pForwardCompat != 0);
			}*/
		private:
			ULONG_PTR Magic() const
				{ return ExceptionInformation[0]; }
		};

		class BasicRegistrationNode : public __Platform::ExceptionFrame
		{
		public:
			BasicRegistrationNode(void * pHandler):
				ExceptionFrame(pHandler)
			{
			}

			void * GetStack() const
				{ return *((void**)this - 1); }
			void SetStack(void * val) const
				{ *((void**)this - 1) = val; }
		private:
			BasicRegistrationNode(const BasicRegistrationNode &);
			BasicRegistrationNode & operator=(const BasicRegistrationNode &);
		};

		class RegistrationNode : public BasicRegistrationNode
		{
		public:
			void * GetStackFrame() const
				{ return (void*)&m_ebp; }
			int state;
		private:
			void * m_ebp;

			//not to be implemented
			RegistrationNode();
			RegistrationNode(const RegistrationNode &);
			RegistrationNode & operator=(const RegistrationNode &);
		};

		

		//may be used for _set_se_trnslator stuff that we do not support
		/*class TranslatorGuardRN : public BasicRegistrationNode
		{
		private:
			GuardDescriptor m_Descriptor; 
			RegistrationNode * m_pMarkerRN;
			void * m_pContinue;
			void * m_ESP;
			void * m_EBP;
			int m_DidUnwind;

			//not to be implemented
			TranslatorGuardRN(const TranslatorGuardRN &);
			TranslatorGuardRN & operator=(const TranslatorGuardRN &);
		};*/


#pragma pack(pop)

		__declspec(noreturn) void inconsistency();

	}

} // namespace __Compiler


#endif /* __COMPILER_EXCEPTION_HANDLING_H */
