/*

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __COMPILER_PROCESS_DATA_H
#define __COMPILER_PROCESS_DATA_H

#include "compiler.h"
#include "atexit.h"
#include "terminate.h"
#include "thread_data.h"
#include "new_handler.h"


namespace __Compiler
{
	class ProcessData 
	{
	public:
		__CRUNTINY_EXPORT static void __CRUNTINY_CRTCALL Init();
		__CRUNTINY_EXPORT static void __CRUNTINY_CRTCALL Term();
		__CRUNTINY_EXPORT static ProcessData & __CRUNTINY_CRTCALL GetInstance();

		AtExit & AtExitList()
			{ return m_AtExitList; }
		Terminate & TerminateHandler()
			{ return m_TerminateHandler; }
		NewHandler & OpNewHandler()
			{ return m_NewHandler; }
		__Platform::Mutex & TypeInfoMutex()
			{ return m_TypeInfoMutex; }
#ifndef _MT
		ThreadData & SingleThreadData()
		{
			return m_SingleThreadData;
		}
#else
		__Platform::TlsData<ThreadData> & ThreadTlsData()
		{
			return m_ThreadTlsData;
		}
#endif
	private:
		ProcessData():
		   m_RefCount(1)
		{}
		~ProcessData()
		{
			m_AtExitList.Invoke();
		}

	// Members
	private:
		unsigned m_RefCount;
		AtExit m_AtExitList;
		Terminate m_TerminateHandler;
		NewHandler m_NewHandler;
		__Platform::Mutex m_TypeInfoMutex;
#ifndef _MT
		ThreadData m_SingleThreadData;
#else
		__Platform::TlsData<ThreadData> m_ThreadTlsData;
#endif

		static ProcessData * s_pProcessData;

	// Not to be implemented
	private:
		ProcessData(ProcessData const &);
		ProcessData & operator =(ProcessData const &);
	};

} // namespace __Compiler

#endif /* __COMPILER_PROCESS_DATA_H */
