/*

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __COMPILER_RTTI_H
#define __COMPILER_RTTI_H

#include "compiler.h"


using namespace __Compiler;


namespace __Compiler
{
	namespace RTTI
	{
		
		/*  these types are known to compiler without declaration
		the declaration is here for convenience

		struct _PMD
		{
			int mdisp;
			int pdisp;
			int vdisp;
		};

		//this is just a binary image of type_info
		struct _TypeDescriptor
		{
			const void * pVFTable;
			void * spare;	
			char name[1];
		};
		*/

		typedef ::_PMD PMD;
		typedef ::_TypeDescriptor TypeDescriptor;

		//_s_RTTIBaseClassDescriptor
		struct BaseClassDescriptor
		{
			TypeDescriptor * pTypeDescriptor;
			unsigned long numContainedBases;
			PMD where;
			unsigned long attributes;
		};

#pragma warning(push)
#pragma warning(disable:4200) //non-standard var size structure
		//_s_RTTIBaseClassArray
		struct BaseClassArray
		{
			const BaseClassDescriptor * arrayOfBaseClassDescriptors[];
		};
#pragma warning(pop)

		//_s_RTTIClassHierarchyDescriptor
		struct ClassHierarchyDescriptor
		{
			unsigned long signature;
			unsigned long attributes;
			unsigned long numBaseClasses;
			const BaseClassArray * pBaseClassArray;
		};

		//_s_RTTICompleteObjectLocator
		struct CompleteObjectLocator
		{
			unsigned long signature;
			unsigned long offset;
			unsigned long cdOffset;
			TypeDescriptor * pTypeDescriptor;
			const ClassHierarchyDescriptor * pClassDescriptor;
		};

		void * AdjustPointer(void * pThis, const PMD & pmd);
		bool operator==(const TypeDescriptor & lhs, const TypeDescriptor & rhs);
	}
}



#endif //__COMPILER_RTTI_H





