/*
 *
 *
 * Updated: 28th February 2003

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __COMPILER_SEH_H
#define __COMPILER_SEH_H

#include "compiler.h"


namespace __Compiler
{
	namespace SEH
	{
		extern const DWORD EH_NONCONTINUABLE;
		extern const DWORD EH_UNWINDING;
		extern const DWORD EH_EXIT_UNWIND;
		extern const DWORD EH_STACK_INVALID;
        extern const DWORD EH_NESTED_CALL;
		extern const DWORD EH_UNKNOWN1;
		extern const DWORD EH_UNKNOWN2;
		extern const DWORD EH_UNKNOWN3;

		typedef ::CONTEXT Context;

		class ExceptionRecord : public ::EXCEPTION_RECORD
		{
		public:
			bool IsUnwinding() const
				{ return (ExceptionFlags & (EH_UNWINDING | EH_EXIT_UNWIND)) != 0; }
		};

		struct ScopeTableEntry
		{
			int previousTryLevel;
			DWORD (*lpfnFilter)();
			DWORD (*lpfnHandler)();
		};

		class ExceptionFrame : public __Platform::ExceptionFrame
		{
		public:
			void * EBP() const
				{ return (void*)&m_ebp; }
			const ScopeTableEntry & CurrentEntry() const
				{ return m_pScopeTable[m_TryLevel]; }
			const ScopeTableEntry & Entry(int level) const
				{ return m_pScopeTable[level]; }
			int TryLevel() const
				{ return m_TryLevel; }
			void SetTryLevel(int val)
				{ m_TryLevel = val; }
			EXCEPTION_POINTERS * GetExceptionPointers() const
				{ return *((EXCEPTION_POINTERS**)this - 1); }
			void SetExceptionPointers(EXCEPTION_POINTERS * val) const
				{ *((EXCEPTION_POINTERS**)this - 1) = val; }
		private:
			ScopeTableEntry * m_pScopeTable;
			int m_TryLevel;
			void * m_ebp;

			//Not to be implemented
			ExceptionFrame();
			ExceptionFrame(const ExceptionFrame &);
			ExceptionFrame & operator=(const ExceptionFrame &);
		};

		class UnwindFrame : public __Platform::ExceptionFrame
		{
		public:
			UnwindFrame(void * pHandler, int tryLevel, SEH::ExceptionFrame * pOriginalFrame):
				ExceptionFrame(pHandler),
				m_TryLevel(tryLevel),
				m_pOriginalFrame(pOriginalFrame)
			{
			}
			int TryLevel() const
				{ return m_TryLevel; }
			void SetTryLevel(int val)
				{ m_TryLevel = val; }
			const SEH::ExceptionFrame * OriginalFrame() const
				{ return m_pOriginalFrame; }
		private:
			int m_TryLevel;
			SEH::ExceptionFrame * m_pOriginalFrame;

			UnwindFrame(const UnwindFrame &);
			UnwindFrame & operator=(const UnwindFrame &);
		};
	}

} // namespace __Compiler


#endif /* __COMPILER_SEH_H */
