/*

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <crtcore.h>
#include "compiler.h"
#include <process.h>
#include "thread.h"
#include "thread_data.h"
#include "map_errno.h"

namespace __Compiler
{

#ifdef _MT
	uintptr_t Thread::Run(CStartAddress start_address,
						  unsigned stack_size, 
						  void * arglist)
	{
		m_CStartAddress = start_address;
		m_ArgList = arglist;
		unsigned id = 0;
		uintptr_t handle = __Platform::CreateThread(0, 
													stack_size, 
													Startup, 
													this, 
													0, 
													&id);
		if (handle == 0)
		{
			MapErrno(GetLastError());
			return uintptr_t(-1);
		}
		return handle;
	}

	uintptr_t Thread::Run(void * security, 
						  unsigned stack_size,
						  WinStartAddress start_address, 
						  void * arglist, 
						  unsigned initflag, 
						  unsigned * threadid)
	{
		m_WinStartAddress = start_address;
		m_ArgList = arglist;
		uintptr_t handle = __Platform::CreateThread(
										security, 
										stack_size, 
										Startup, 
										this, 
										initflag, 
										threadid);
		if (handle == 0)
		{
			MapErrno(GetLastError());
		}
		return handle;
	}

	unsigned long __stdcall Thread::Startup(void * arglist)
	{
		Thread * pThis = static_cast<Thread *>(arglist);
		Thread CopyOfThis(*pThis);
		delete pThis;

		//ensure we have a thread data
		ThreadData		* ptdata = ThreadData::GetInstance();
		unsigned long	ret = 0;

		if (CopyOfThis.m_WinStartAddress)
		{
			ret = CopyOfThis.m_WinStartAddress(CopyOfThis.m_ArgList);
		}
		else
		{
			CopyOfThis.m_CStartAddress(CopyOfThis.m_ArgList);
		}

		Cleanup(ptdata, ret);

#ifndef __CRUNTINY_DECLSPEC_NORETURN_SUPPORTED
		return ret;
#endif /* !__CRUNTINY_DECLSPEC_NORETURN_SUPPORTED */
	}

	__CRUNTINY_DECLSPEC_NORETURN
	void Thread::Cleanup(ThreadData * ptdata, unsigned retval)
	{
		if (ptdata)
		{
			ptdata->DestroyInstance();
		}

		__Platform::ExitThread(retval);
	}

#endif //_MT

} // namespace __Compiler
