/*

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __COMPILER_THREAD_DATA_H
#define __COMPILER_THREAD_DATA_H

#include "compiler.h"
#include <exception>
#include "eh_context.h"
#include "eh_guard_descriptor.h"

namespace __Compiler
{
	class ThreadData
	{
	friend class ProcessData;
	public:
		//get the instance of thread data. If it does not exist
		//create it
		__CRUNTINY_EXPORT static ThreadData * __CRUNTINY_CRTCALL GetInstance();
#ifdef _MT
		__CRUNTINY_EXPORT void DestroyInstance();
#endif

		int & Errno()
			{ return m_Errno; }

		EH::ExceptionContext & CurrentException()
			{ return m_CurrentException; }

		EH::GuardDescriptor::FrameInfo * & FrameInfoChain()
			{ return m_pFrameInfoChain; }

		int & ProcessingThrow()
			{ return m_ProcessingThrow; }
		
	private:
		ThreadData();
		~ThreadData() {};

		int m_Errno;
		EH::ExceptionContext m_CurrentException;
		EH::GuardDescriptor::FrameInfo * m_pFrameInfoChain;
		int m_ProcessingThrow;
	};

} // namespace __Compiler


#endif /* __COMPILER_THREAD_DATA_H */
