/*

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <crtcore.h>
#include "compiler.h"
#include <typeinfo.h>
#include "process_data.h"
#include <dbghelp.h>

using namespace __Compiler;


char * __unDName(char * /*outputString*/ , 
				 const char * name, 
				 int /*maxStringLength*/ , 
				 void * (* pAlloc )(size_t) , 
				 void (* /*pFree*/ )(void *), 
				 unsigned short /*disableFlags*/)
{
	const size_t size = strlen(name) + 1;
	char * ret = (char*)pAlloc(size);
	if (ret)
		strcpy(ret, name);
	return ret;
}


__CRUNTINY_EXPORT char const * type_info::name() const
{
	if (!m_Data)
	{
		char * pTmpUndName = __unDName(0, &m_DecoratedName[1], 0, malloc, free, 
			UNDNAME_NO_ARGUMENTS | UNDNAME_32_BIT_DECODE);
		if (!pTmpUndName)
			return 0;

		char * p = pTmpUndName + strlen(pTmpUndName) - 1;
		while(*p == ' ')
		{
			*p-- = 0;
		}

		SYNCHRONIZE_ON(ProcessData::GetInstance().TypeInfoMutex())
			m_Data = malloc(p - pTmpUndName + 1);
			if (m_Data)
				memcpy(m_Data, pTmpUndName, p - pTmpUndName + 1);
		END_SYNCHRONIZE

		free(pTmpUndName);
	}

	return (const char*)m_Data;
}
