/*
 * Updated: 12th December 2003

Copyright (c) 2003, CRunTiny.org
Copyright (c) 2001, Synesis Software Pty Ltd (http://synesis.com.au)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <crtcore.h>
#include "cmdline.h"

namespace __Platform
{
	/* This implementation derives from the Synesis Software implementation, 
	 * specifically MLCmdArg.c v1.6.3.33.
	 *
	 * Please concurrently report any bugs/issues to software@synesis.com.au
	 * in order to facilitate our maintaining correct versions of the code.
	 */

	#define	MS_COMPAT_QUOTE_ESC	/* Adopts Microsoft behaviour for quotes */
	/* #define	_GIVE_THE_BEST_IN_ERROR */

	typedef	int				Boolean;
	typedef	int				Int;
	typedef	size_t			Size;
	typedef	void			*PVoid;
	typedef	void			*PIAllocator;
	typedef	unsigned		UInt32;
	typedef	char			AChar;
	typedef	wchar_t			WChar;
	typedef	char			*PAChar;
	typedef	wchar_t			*PWChar;
	typedef	char const		*PCAChar;
	typedef	wchar_t const	*PCWChar;
	typedef	char			**PPAChar;
	typedef	wchar_t			**PPWChar;
	typedef	char const		**PPCAChar;
	typedef	wchar_t const	**PPCWChar;

	#define	StrLenA				strlen
	#define	StrLenW				wcslen
	#define	StrCpyA				strcpy
	#define	StrCpyW				wcscpy
	#define	SyCastConst(t, v)	const_cast<t>(v)
	#define	SyCastStatic(t, v)	static_cast<t>(v)
#ifdef _DEBUG
	#define	SyAssert(x)			((void)(x))	/* TODO: Make this into a real assert, once we've added one to CRunTiny */
#else /* ? _DEBUG */
	#define	SyAssert(x)
#endif /* _DEBUG */

	namespace
	{
		inline Boolean __isspace(int c) /* TODO: Link this up with isspace from CRunTiny/CStdioTiny */
		{
			return c == ' ' || c == '\t' || (c >= '\r' && c <= '\n');
		}

		inline void const *PtrByteOffset(void const *p, ptrdiff_t n)
		{
			return static_cast<void *>(static_cast<char*>(const_cast<void *>(p)) + n);
		}
	}

#pragma warning(disable : 4127)

	__CRUNTINY_EXPORT
	int PrepareCommandLineArgsA(char **&argv)
	{
		typedef enum State
		{
				whiteSpace	=	0			// Whitespace
			,	clearQuotePair				// Quote pair from whitespace
			,	clearQuote					// Open quote from whitespace
			,	clearQuoteEscape			// Open quote from whitespace
			,	clearEscape					// Open quote from whitespace
			,	openText					// Open text
			,	openTextQuote				// Open text
			,	openTextQuotePair			// Open text
			,	openTextEscape				// Open text
			,	openTextEscaped				// Open text
			,	quotedText					// Quoted text
			,	quotedTextQuote				// Quoted text double quote	
			,	quotedTextQuotePair			// Quoted text double quote
			,	quotedTextEscape			// Quoted text escape character
			,	quotedTextEscaped			// Quoted text escaped
		} State;

		int			argc;
		char		*lpszArgs	=	::GetCommandLineA();
		size_t		len			=	StrLenA(lpszArgs);
		PCAChar		begin		=	lpszArgs;
		PCAChar		end			=	begin + len;
		State		state;
		Boolean		bRet;

		for(argc = 0, state = whiteSpace, bRet = true; bRet && begin < end; ++begin)
		{
			const AChar	c = *begin;

			switch(state)
			{
				case	whiteSpace:
					if(__isspace(c))
					{
					}
					else if(c == '"')
					{
						state = clearQuote;
					}
					else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
							(begin + 1 < end) &&
							(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
							1)
					{
						state = clearEscape;
					}
					else
					{
						state = openText;
					}
					break;
				/// Clear text ///
				case	clearQuote:
					if(c == '"')
					{
						state = clearQuotePair;
					}
					else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
							(begin + 1 < end) &&
							(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
							1)
					{
						state = clearQuoteEscape;
					}
					else
					{
						state = quotedText;
					}
					break;
				case	clearQuotePair:
					if(__isspace(c))
					{
						++argc;
						state = whiteSpace;
					}
					else if(c == '"')
					{
						state = openText;
					}
					else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
							(begin + 1 < end) &&
							(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
							1)
					{
						state = openTextEscape;
					}
					else
					{
						state = openText;
					}
					break;
				case	clearQuoteEscape:
					state = quotedText;
					break;
				case	clearEscape:
					if(__isspace(c))
					{
						++argc;
						state = whiteSpace;
					}
					else if(c == '"')
					{
						state = openTextEscaped;
					}
					else
					{
						state = openText;
					}
					break;
				/// Open text ///
				case	openText:
					if(__isspace(c))
					{
						++argc;
						state = whiteSpace;
					}
					else if(c == '"')
					{
						state = openTextQuote;
					}
					else if(c == '\\')
					{
						state = openTextEscape;
					}
					else
					{
						state = openText;
					}
					break;
				case	openTextQuote:
					if(c == '"')
					{
						state = openTextQuotePair;
					}
					else
					{
						state = quotedText;
					}
					break;
				case	openTextQuotePair:
					if(__isspace(c))
					{
						++argc;
						state = whiteSpace;
					}
					else
					{
						state = openText;
					}
					break;
				case	openTextEscape:
					if(__isspace(c))
					{
						++argc;
						state = whiteSpace;
					}
					else
					{
						state = openText;
					}
					break;
				case	openTextEscaped:
					if(__isspace(c))
					{
						state = whiteSpace;
					}
					else if(c == '"')
					{
						state = openTextQuote;
					}
					else if(c == '\\')
					{
						state = openTextEscape;
					}
					else
					{
						state = openText;
					}
					break;
				/// Quoted text ///
				case	quotedText:
					if(c == '"')
					{
						state = quotedTextQuote;
					}
					else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
							(begin + 1 < end) &&
							(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
							1)
					{
						state = quotedTextEscape;
					}
					break;
				case	quotedTextQuote:
					if(__isspace(c))
					{
						++argc;
						state = whiteSpace;
					}
					else if(c == '"')
					{
						state = quotedTextQuotePair;
					}
					else if(c == '\\')
					{
						state = openTextEscape;
					}
					else
					{
						state = openText;
					}
					break;
				case	quotedTextQuotePair:
					state = quotedText;
					break;
				case	quotedTextEscape:
					state = quotedTextEscaped;
					break;
				case	quotedTextEscaped:
					state = quotedText;
					break;
				default:
					SyAssert(0);
					break;
			}
		}

		switch(state)
		{
			case	whiteSpace:
				break;
			case	clearQuote:
			case	clearQuotePair:
			case	clearQuoteEscape:
			case	clearEscape:
			case	openText:
			case	openTextEscape:
			case	openTextEscaped:
			case	openTextQuote:
			case	openTextQuotePair:
			case	quotedText:
			case	quotedTextQuote:
			case	quotedTextQuotePair:
				++argc;
				break;
			default:
				SyAssert(0);
				break;
		}

		if( !bRet ||
			argc < 1)
		{
			argv = NULL;
			argc = 0;
		}
		else
		{
			/* Allocate memory. Has format ator | argc | argv[] | contents */
			Size	siString	=	sizeof(AChar) * (2 + len);
			Size	siArgs		=	argc * sizeof(PPCAChar);
			Size	siTotal		=	sizeof(PIAllocator) + sizeof(UInt32) + siArgs + siString;
			AChar	*block		=	new AChar[siTotal];
			PPAChar	ppArgs		=	reinterpret_cast<PPAChar>(block);
			PAChar	buffer		=	(PAChar)PtrByteOffset(block, siArgs);
			PAChar	begin		=	buffer;
			PAChar	end			=	begin + len;

			PAChar	copy_dest	=	NULL;
			bool	bCopy		=	false;

			/* Copy in the buffer. */
			StrCpyA(buffer, lpszArgs);
			buffer[len + 1] = '\0';

			argv = ppArgs;

			/* Copy the argument count. */
#ifdef _DEBUG
			int	cArgs = argc;
#endif /* _DEBUG */
			argc = 0;

			/* Now scan again, and this time insert the null terminators. */
			for(state = whiteSpace, copy_dest = NULL; begin < end; ++begin)
			{
				const AChar	c		= *begin;

				bCopy = false;

				switch(state)
				{
					case	whiteSpace:
						if(__isspace(c))
						{
							// Nothing to do, stay in whitespace
						}
						else if(c == '"')
						{
							// Move into the start of a quote
							state = clearQuote;
						}
						else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
								(begin + 1 < end) &&
								(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
								1)
						{
							// When in whitespace sequences, the backslash is recognised as an escape operator
							*ppArgs = begin + 1;	// Mark that this may be the start of a sequence
							state = clearEscape;
						}
						else
						{
							// Recognised start of a non-quoted section
							copy_dest = NULL;
							++argc;
							*ppArgs++ = begin;
							state = openText;
						}
						break;
					case	clearQuote:
						if(c == '"')
						{
							// Now have received two contiguous quotes after some whitespace
							state = clearQuotePair;
						}
						else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
								(begin + 1 < end) &&
								(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
								1)
						{
							// When in quoted sequences, the backslash is recognised as an escape operator
							*ppArgs = begin + 1;	// Mark that this may be the start of a sequence
							state = clearQuoteEscape;
						}
						else
						{
							// Recognised start of a quoted sequence, so record current position
							// which skips the preceeding quote
							copy_dest = NULL;
							++argc;
							*ppArgs++ = begin;
							state = quotedText;
						}
						break;
					case	clearQuotePair:
						if(__isspace(c))
						{
							// Whitespace after a quote pair that was itself preceeded by whitespace
							// denotes an empty argument, so record that with the empty string
							copy_dest = NULL;
							++argc;
							*ppArgs++ = "";
							state = whiteSpace;
						}
	#if 1
						else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
								(begin + 1 < end) &&
								(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
								1)
						{
							state = openTextEscape;
							if(copy_dest == NULL)
							{
								copy_dest = begin;
							}
						}
	#endif /* 0 */
						else
						{
							// Any text character immediately following a whitespace preceeded-quote pair
							// can be used as the start of that argument
							copy_dest = NULL;
							++argc;
							*ppArgs++ = begin;
							state = openText;
						}
						break;
					case	clearQuoteEscape:
						// When in a whitespace preceeded-quote/escape, everything is taken to start the
						// next argument
						copy_dest = NULL;
						++argc;
						*ppArgs++ = begin;
						state = quotedText;
						break;
					case	clearEscape:
						if(__isspace(c))
						{
							// Whitespace characters mark the end of the current argument
							*begin = '\0';
							if(copy_dest != NULL)
							{
								*copy_dest = '\0';
								copy_dest = NULL;
							}
							state = whiteSpace;
						}
						else if(c == '"')
						{
							++argc;
							++ppArgs;
							state = openTextEscaped;
						}
						else
						{
							bCopy = true;
							state = openText;
						}
						break;
					/// Open text ///
					case	openText:
						if(__isspace(c))
						{
							// Whitespace characters mark the end of the current argument
							*begin = '\0';
							if(copy_dest != NULL)
							{
								*copy_dest = '\0';
								copy_dest = NULL;
							}
							state = whiteSpace;
						}
						else if(c == '"')
						{
							// A quote in an open-text section marks a quoted section to be concatenated
							// with the open section, so move into quotedTextQuote state
							state = openTextQuote;
							if(copy_dest == NULL)
							{
								copy_dest = begin;
							}
						}
						else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
								(begin + 1 < end) &&
								(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
								1)
						{
							state = openTextEscape;
							if(copy_dest == NULL)
							{
								copy_dest = begin;
							}
						}
						else
						{
							bCopy = true;
							state = openText;
						}
						break;
					case	openTextQuote:
						if(c == '"')
						{
							state = openTextQuotePair;
						}
						else
						{
							bCopy = true;
							state = quotedText;
						}
						break;
					case	openTextQuotePair:
						if(__isspace(c))
						{
							// Whitespace characters mark the end of the current argument
							*begin = '\0';
							if(copy_dest != NULL)
							{
								*copy_dest = '\0';
								copy_dest = NULL;
							}
							state = whiteSpace;
						}
						else
						{
							bCopy = true;
							state = openText;
						}
						break;
					case	openTextEscape:
						if(__isspace(c))
						{
							// Whitespace characters mark the end of the current argument
							*begin = '\0';
							if(copy_dest != NULL)
							{
								*copy_dest = '\0';
								copy_dest = NULL;
							}
							state = whiteSpace;
						}
						else
						{
							bCopy = true;
							state = openText;
						}
						break;
					case	openTextEscaped:
						// Every escaped character is added to the argument
						bCopy = true;
						state = openText;
						break;
					/// Quoted text ///
					case	quotedText:
						if(c == '"')
						{
							state = quotedTextQuote;
							if(copy_dest == NULL)
							{
								copy_dest = begin;
							}
						}
						else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
								(begin + 1 < end) &&
								(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
								1)
						{
							// 
							state = quotedTextEscape;
							if(copy_dest == NULL)
							{
								copy_dest = begin;
							}
						}
						else
						{
							bCopy = true;
						}
						break;
					case	quotedTextQuote:
						if(__isspace(c))
						{
							*(begin - 1) = '\0';
							if(copy_dest != NULL)
							{
								*copy_dest = '\0';
								copy_dest = NULL;
							}
							// Returns to whitespace
							state = whiteSpace;
						}
						else if(c == '"')
						{
							// two contiguous quotes are either a precursor to three quotes, or mark the
							// end and beginning of two contiguous quoted texts that will be concatenated
							// into one argument

							// There's a real funny quirk here, whereby we need to look ahead.
							if(begin + 1 < end)
							{
								AChar	cNext	=	*(begin + 1);

								if(	cNext != '"' &&
									!__isspace(cNext))
								{
									bCopy = true;
								}
							}

							state = quotedTextQuotePair;
						}
						else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
								(begin + 1 < end) &&
								(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
								1)
						{
							state = openTextEscape;
							if(copy_dest == NULL)
							{
								copy_dest = begin;
							}
						}
						else
						{
							// Any other character after a quote is included in the argument and the
							// state reverts to openText
							bCopy = true;
							state = openText;
						}
						break;
					case	quotedTextQuotePair:
						// The next character is included in the quoted text
						bCopy = true;
						state = quotedText;
						break;
					case	quotedTextEscape:
						if(begin + 1 < end)
						{
							AChar	cNext	=	*(begin + 1);

							if(	cNext != '"' &&
								!__isspace(cNext))
							{
								bCopy = true;
							}
						}
						state = quotedTextEscaped;
						break;
					case	quotedTextEscaped:
						// Every escaped character is added to the argument
						bCopy = true;
						state = quotedText;
						break;
					default:
						SyAssert(0);
						break;
				}

				SyAssert(copy_dest == NULL || copy_dest >= buffer);
				SyAssert(copy_dest == NULL || copy_dest <= begin);

				if(bCopy)
				{
					if(copy_dest != NULL)
					{
						*copy_dest++ = *begin;
					}
				}
			}

			switch(state)
			{
				case	whiteSpace:
					break;
				case	clearQuote:
				case	clearQuotePair:
					++argc;
					*ppArgs++ = "";
					break;
				case	clearQuoteEscape:
				case	clearEscape:
					++argc;
					++ppArgs;
					break;
				case	openText:
				case	quotedText:
					if(copy_dest == NULL)
					{
						bCopy = false;
					}
					break;
				case	openTextQuote:
				case	openTextQuotePair:
				case	quotedTextQuotePair:
					if(copy_dest != NULL)
					{
						bCopy = true;
					}
					break;
				case	openTextEscape:
					break;
				case	quotedTextQuote:
					if(copy_dest == NULL)
					{
						copy_dest = begin - 1;
					}
					bCopy = true;
					break;
				default:
					SyAssert(0);
					break;
			}

			SyAssert(cArgs == argc);
			SyAssert((ppArgs - reinterpret_cast<PPAChar>(block)) == argc);

			// Need to end the quote here
			SyAssert(copy_dest == NULL || copy_dest >= buffer);
			SyAssert(copy_dest == NULL || copy_dest <= begin);

			if(bCopy)
			{
				SyAssert(copy_dest != NULL);

				*copy_dest++ = '\0';
			}

			/* Now check all the arguments to see if they have any prefixed
			* whitespace. If any do, then they are just shifted along to
			* the first non-whitespace.
			*/
	#ifdef _GIVE_THE_BEST_IN_ERROR
			if(bRet)
			{
				Int		i;
				PAChar	p;

				for(i = 0; i < argc; ++i)
				{
					p = (PAChar)(*argv)[i];

					while(__isspace(*p))
					{
						++p;
					}

					(*argv)[i] = p;
				}
			}
	#endif /* _GIVE_THE_BEST_IN_ERROR */
		}

		return argc;
	}

	__CRUNTINY_EXPORT
	void ReleaseCommandLineArgsA(char **argv)
	{
		AChar	*pArgv = reinterpret_cast<AChar*>(argv);

		delete [] pArgv;
	}

	__CRUNTINY_EXPORT
	int PrepareCommandLineArgsW(wchar_t **&argv)
	{
		typedef enum State
		{
				whiteSpace	=	0			// Whitespace
			,	clearQuotePair				// Quote pair from whitespace
			,	clearQuote					// Open quote from whitespace
			,	clearQuoteEscape			// Open quote from whitespace
			,	clearEscape					// Open quote from whitespace
			,	openText					// Open text
			,	openTextQuote				// Open text
			,	openTextQuotePair			// Open text
			,	openTextEscape				// Open text
			,	openTextEscaped				// Open text
			,	quotedText					// Quoted text
			,	quotedTextQuote				// Quoted text double quote	
			,	quotedTextQuotePair			// Quoted text double quote
			,	quotedTextEscape			// Quoted text escape character
			,	quotedTextEscaped			// Quoted text escaped
		} State;

		int			argc;
		WChar		*lpszArgs	=	::GetCommandLineW();
		size_t		len			=	StrLenW(lpszArgs);
		PCWChar		begin		=	lpszArgs;
		PCWChar		end			=	begin + len;
		State		state;
		Boolean		bRet;

		for(argc = 0, state = whiteSpace, bRet = true; bRet && begin < end; ++begin)
		{
			const WChar	c = *begin;

			switch(state)
			{
				case	whiteSpace:
					if(__isspace(c))
					{
					}
					else if(c == '"')
					{
						state = clearQuote;
					}
					else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
							(begin + 1 < end) &&
							(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
							1)
					{
						state = clearEscape;
					}
					else
					{
						state = openText;
					}
					break;
				/// Clear text ///
				case	clearQuote:
					if(c == '"')
					{
						state = clearQuotePair;
					}
					else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
							(begin + 1 < end) &&
							(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
							1)
					{
						state = clearQuoteEscape;
					}
					else
					{
						state = quotedText;
					}
					break;
				case	clearQuotePair:
					if(__isspace(c))
					{
						++argc;
						state = whiteSpace;
					}
					else if(c == '"')
					{
						state = openText;
					}
					else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
							(begin + 1 < end) &&
							(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
							1)
					{
						state = openTextEscape;
					}
					else
					{
						state = openText;
					}
					break;
				case	clearQuoteEscape:
					state = quotedText;
					break;
				case	clearEscape:
					if(__isspace(c))
					{
						++argc;
						state = whiteSpace;
					}
					else if(c == '"')
					{
						state = openTextEscaped;
					}
					else
					{
						state = openText;
					}
					break;
				/// Open text ///
				case	openText:
					if(__isspace(c))
					{
						++argc;
						state = whiteSpace;
					}
					else if(c == '"')
					{
						state = openTextQuote;
					}
					else if(c == '\\')
					{
						state = openTextEscape;
					}
					else
					{
						state = openText;
					}
					break;
				case	openTextQuote:
					if(c == '"')
					{
						state = openTextQuotePair;
					}
					else
					{
						state = quotedText;
					}
					break;
				case	openTextQuotePair:
					if(__isspace(c))
					{
						++argc;
						state = whiteSpace;
					}
					else
					{
						state = openText;
					}
					break;
				case	openTextEscape:
					if(__isspace(c))
					{
						++argc;
						state = whiteSpace;
					}
					else
					{
						state = openText;
					}
					break;
				case	openTextEscaped:
					if(__isspace(c))
					{
						state = whiteSpace;
					}
					else if(c == '"')
					{
						state = openTextQuote;
					}
					else if(c == '\\')
					{
						state = openTextEscape;
					}
					else
					{
						state = openText;
					}
					break;
				/// Quoted text ///
				case	quotedText:
					if(c == '"')
					{
						state = quotedTextQuote;
					}
					else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
							(begin + 1 < end) &&
							(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
							1)
					{
						state = quotedTextEscape;
					}
					break;
				case	quotedTextQuote:
					if(__isspace(c))
					{
						++argc;
						state = whiteSpace;
					}
					else if(c == '"')
					{
						state = quotedTextQuotePair;
					}
					else if(c == '\\')
					{
						state = openTextEscape;
					}
					else
					{
						state = openText;
					}
					break;
				case	quotedTextQuotePair:
					state = quotedText;
					break;
				case	quotedTextEscape:
					state = quotedTextEscaped;
					break;
				case	quotedTextEscaped:
					state = quotedText;
					break;
				default:
					SyAssert(0);
					break;
			}
		}

		switch(state)
		{
			case	whiteSpace:
				break;
			case	clearQuote:
			case	clearQuotePair:
			case	clearQuoteEscape:
			case	clearEscape:
			case	openText:
			case	openTextEscape:
			case	openTextEscaped:
			case	openTextQuote:
			case	openTextQuotePair:
			case	quotedText:
			case	quotedTextQuote:
			case	quotedTextQuotePair:
				++argc;
				break;
			default:
				SyAssert(0);
				break;
		}

		if( !bRet ||
			argc < 1)
		{
			argv = NULL;
			argc = 0;
		}
		else
		{
			/* Allocate memory. Has format ator | argc | argv[] | contents */
			Size	siString	=	sizeof(WChar) * (2 + len);
			Size	siArgs		=	argc * sizeof(PPCWChar);
			Size	siTotal		=	sizeof(PIAllocator) + sizeof(UInt32) + siArgs + siString;
			WChar	*block		=	new WChar[siTotal];
			PPWChar	ppArgs		=	reinterpret_cast<PPWChar>(block);
			PWChar	buffer		=	(PWChar)PtrByteOffset(block, siArgs);
			PWChar	begin		=	buffer;
			PWChar	end			=	begin + len;

			PWChar	copy_dest	=	NULL;
			bool	bCopy		=	false;

			/* Copy in the buffer. */
			StrCpyW(buffer, lpszArgs);
			buffer[len + 1] = '\0';

			argv = ppArgs;

			/* Copy the argument count. */
#ifdef _DEBUG
			int	cArgs = argc;
#endif /* _DEBUG */
			argc = 0;

			/* Now scan again, and this time insert the null terminators. */
			for(state = whiteSpace, copy_dest = NULL; begin < end; ++begin)
			{
				const WChar	c		= *begin;

				bCopy = false;

				switch(state)
				{
					case	whiteSpace:
						if(__isspace(c))
						{
							// Nothing to do, stay in whitespace
						}
						else if(c == '"')
						{
							// Move into the start of a quote
							state = clearQuote;
						}
						else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
								(begin + 1 < end) &&
								(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
								1)
						{
							// When in whitespace sequences, the backslash is recognised as an escape operator
							*ppArgs = begin + 1;	// Mark that this may be the start of a sequence
							state = clearEscape;
						}
						else
						{
							// Recognised start of a non-quoted section
							copy_dest = NULL;
							++argc;
							*ppArgs++ = begin;
							state = openText;
						}
						break;
					case	clearQuote:
						if(c == '"')
						{
							// Now have received two contiguous quotes after some whitespace
							state = clearQuotePair;
						}
						else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
								(begin + 1 < end) &&
								(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
								1)
						{
							// When in quoted sequences, the backslash is recognised as an escape operator
							*ppArgs = begin + 1;	// Mark that this may be the start of a sequence
							state = clearQuoteEscape;
						}
						else
						{
							// Recognised start of a quoted sequence, so record current position
							// which skips the preceeding quote
							copy_dest = NULL;
							++argc;
							*ppArgs++ = begin;
							state = quotedText;
						}
						break;
					case	clearQuotePair:
						if(__isspace(c))
						{
							// Whitespace after a quote pair that was itself preceeded by whitespace
							// denotes an empty argument, so record that with the empty string
							copy_dest = NULL;
							++argc;
							*ppArgs++ = L"";
							state = whiteSpace;
						}
	#if 1
						else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
								(begin + 1 < end) &&
								(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
								1)
						{
							state = openTextEscape;
							if(copy_dest == NULL)
							{
								copy_dest = begin;
							}
						}
	#endif /* 0 */
						else
						{
							// Any text character immediately following a whitespace preceeded-quote pair
							// can be used as the start of that argument
							copy_dest = NULL;
							++argc;
							*ppArgs++ = begin;
							state = openText;
						}
						break;
					case	clearQuoteEscape:
						// When in a whitespace preceeded-quote/escape, everything is taken to start the
						// next argument
						copy_dest = NULL;
						++argc;
						*ppArgs++ = begin;
						state = quotedText;
						break;
					case	clearEscape:
						if(__isspace(c))
						{
							// Whitespace characters mark the end of the current argument
							*begin = '\0';
							if(copy_dest != NULL)
							{
								*copy_dest = '\0';
								copy_dest = NULL;
							}
							state = whiteSpace;
						}
						else if(c == '"')
						{
							++argc;
							++ppArgs;
							state = openTextEscaped;
						}
						else
						{
							bCopy = true;
							state = openText;
						}
						break;
					/// Open text ///
					case	openText:
						if(__isspace(c))
						{
							// Whitespace characters mark the end of the current argument
							*begin = '\0';
							if(copy_dest != NULL)
							{
								*copy_dest = '\0';
								copy_dest = NULL;
							}
							state = whiteSpace;
						}
						else if(c == '"')
						{
							// A quote in an open-text section marks a quoted section to be concatenated
							// with the open section, so move into quotedTextQuote state
							state = openTextQuote;
							if(copy_dest == NULL)
							{
								copy_dest = begin;
							}
						}
						else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
								(begin + 1 < end) &&
								(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
								1)
						{
							state = openTextEscape;
							if(copy_dest == NULL)
							{
								copy_dest = begin;
							}
						}
						else
						{
							bCopy = true;
							state = openText;
						}
						break;
					case	openTextQuote:
						if(c == '"')
						{
							state = openTextQuotePair;
						}
						else
						{
							bCopy = true;
							state = quotedText;
						}
						break;
					case	openTextQuotePair:
						if(__isspace(c))
						{
							// Whitespace characters mark the end of the current argument
							*begin = '\0';
							if(copy_dest != NULL)
							{
								*copy_dest = '\0';
								copy_dest = NULL;
							}
							state = whiteSpace;
						}
						else
						{
							bCopy = true;
							state = openText;
						}
						break;
					case	openTextEscape:
						if(__isspace(c))
						{
							// Whitespace characters mark the end of the current argument
							*begin = '\0';
							if(copy_dest != NULL)
							{
								*copy_dest = '\0';
								copy_dest = NULL;
							}
							state = whiteSpace;
						}
						else
						{
							bCopy = true;
							state = openText;
						}
						break;
					case	openTextEscaped:
						// Every escaped character is added to the argument
						bCopy = true;
						state = openText;
						break;
					/// Quoted text ///
					case	quotedText:
						if(c == '"')
						{
							state = quotedTextQuote;
							if(copy_dest == NULL)
							{
								copy_dest = begin;
							}
						}
						else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
								(begin + 1 < end) &&
								(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
								1)
						{
							// 
							state = quotedTextEscape;
							if(copy_dest == NULL)
							{
								copy_dest = begin;
							}
						}
						else
						{
							bCopy = true;
						}
						break;
					case	quotedTextQuote:
						if(__isspace(c))
						{
							*(begin - 1) = '\0';
							if(copy_dest != NULL)
							{
								*copy_dest = '\0';
								copy_dest = NULL;
							}
							// Returns to whitespace
							state = whiteSpace;
						}
						else if(c == '"')
						{
							// two contiguous quotes are either a precursor to three quotes, or mark the
							// end and beginning of two contiguous quoted texts that will be concatenated
							// into one argument

							// There's a real funny quirk here, whereby we need to look ahead.
							if(begin + 1 < end)
							{
								WChar	cNext	=	*(begin + 1);

								if(	cNext != '"' &&
									!__isspace(cNext))
								{
									bCopy = true;
								}
							}

							state = quotedTextQuotePair;
						}
						else if(c == '\\' &&
	#ifdef MS_COMPAT_QUOTE_ESC
								(begin + 1 < end) &&
								(begin[1] == '"') &&
	#endif /* MS_COMPAT_QUOTE_ESC */
								1)
						{
							state = openTextEscape;
							if(copy_dest == NULL)
							{
								copy_dest = begin;
							}
						}
						else
						{
							// Any other character after a quote is included in the argument and the
							// state reverts to openText
							bCopy = true;
							state = openText;
						}
						break;
					case	quotedTextQuotePair:
						// The next character is included in the quoted text
						bCopy = true;
						state = quotedText;
						break;
					case	quotedTextEscape:
						if(begin + 1 < end)
						{
							WChar	cNext	=	*(begin + 1);

							if(	cNext != '"' &&
								!__isspace(cNext))
							{
								bCopy = true;
							}
						}
						state = quotedTextEscaped;
						break;
					case	quotedTextEscaped:
						// Every escaped character is added to the argument
						bCopy = true;
						state = quotedText;
						break;
					default:
						SyAssert(0);
						break;
				}

				SyAssert(copy_dest == NULL || copy_dest >= buffer);
				SyAssert(copy_dest == NULL || copy_dest <= begin);

				if(bCopy)
				{
					if(copy_dest != NULL)
					{
						*copy_dest++ = *begin;
					}
				}
			}

			switch(state)
			{
				case	whiteSpace:
					break;
				case	clearQuote:
				case	clearQuotePair:
					++argc;
					*ppArgs++ = L"";
					break;
				case	clearQuoteEscape:
				case	clearEscape:
					++argc;
					++ppArgs;
					break;
				case	openText:
				case	quotedText:
					if(copy_dest == NULL)
					{
						bCopy = false;
					}
					break;
				case	openTextQuote:
				case	openTextQuotePair:
				case	quotedTextQuotePair:
					if(copy_dest != NULL)
					{
						bCopy = true;
					}
					break;
				case	openTextEscape:
					break;
				case	quotedTextQuote:
					if(copy_dest == NULL)
					{
						copy_dest = begin - 1;
					}
					bCopy = true;
					break;
				default:
					SyAssert(0);
					break;
			}

			SyAssert(cArgs == argc);
			SyAssert((ppArgs - reinterpret_cast<PPWChar>(block)) == argc);

			// Need to end the quote here
			SyAssert(copy_dest == NULL || copy_dest >= buffer);
			SyAssert(copy_dest == NULL || copy_dest <= begin);

			if(bCopy)
			{
				SyAssert(copy_dest != NULL);

				*copy_dest++ = '\0';
			}

			/* Now check all the arguments to see if they have any prefixed
			* whitespace. If any do, then they are just shifted along to
			* the first non-whitespace.
			*/
	#ifdef _GIVE_THE_BEST_IN_ERROR
			if(bRet)
			{
				Int		i;
				PAChar	p;

				for(i = 0; i < argc; ++i)
				{
					p = (PAChar)(*argv)[i];

					while(__isspace(*p))
					{
						++p;
					}

					(*argv)[i] = p;
				}
			}
	#endif /* _GIVE_THE_BEST_IN_ERROR */
		}

		return argc;
	}

	__CRUNTINY_EXPORT
	void ReleaseCommandLineArgsW(wchar_t **argv)
	{
		WChar	*pArgv = reinterpret_cast<WChar*>(argv);

		delete [] pArgv;
	}

	__CRUNTINY_EXPORT
	int GetWindowShowState()
	{
		STARTUPINFO sui;

		GetStartupInfo(&sui);

		return (sui.dwFlags & STARTF_USESHOWWINDOW)
					? sui.wShowWindow
					: SW_SHOWDEFAULT;
	}

	__CRUNTINY_EXPORT
	char *GetCommandLineA()
	{
		char *lpszCmdLine = ::GetCommandLineA();

		if(*lpszCmdLine == '\"')
		{
			while(  (*(++lpszCmdLine) != '\"') &&
				(*lpszCmdLine != '\0'))
			{}

			if ( *lpszCmdLine == '\"')
			{
				++lpszCmdLine;
			}
		}
		else
		{
			while(*lpszCmdLine > ' ')
			{
				++lpszCmdLine;
			}
		}

		while(  *lpszCmdLine &&
				(*lpszCmdLine <= ' '))
		{
			++lpszCmdLine;
		}

		return lpszCmdLine;
	}

	__CRUNTINY_EXPORT
	wchar_t *GetCommandLineW()
	{
		wchar_t *lpszCmdLine = ::GetCommandLineW();

		if(*lpszCmdLine == '\"')
		{
			while(  (*(++lpszCmdLine) != '\"') &&
				(*lpszCmdLine != '\0'))
			{}

			if ( *lpszCmdLine == '\"')
			{
				++lpszCmdLine;
			}
		}
		else
		{
			while(*lpszCmdLine > ' ')
			{
				++lpszCmdLine;
			}
		}

		while(  *lpszCmdLine &&
				(*lpszCmdLine <= ' '))
		{
			++lpszCmdLine;
		}

		return lpszCmdLine;
	}

} // namespace __Platform
