/*

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __PLATFORM_WIN32_EXCEPTION_FRAME_H
#define __PLATFORM_WIN32_EXCEPTION_FRAME_H

#include "win32.h"

namespace __Platform
{
	class ExceptionFrame
	{
	/// Constructors
	/// @{
	public:
		ExceptionFrame(void * Handler):
			m_pNext(GetCurrent()),
			m_FrameHandler(Handler)
		{}
	/// @}

	/// Attributes
	/// @{
	public:

	/// @}

	/// Operations
	/// @{
	public:
		ExceptionFrame * Next()
			{ return m_pNext; }
		void * Handler() const
			{ return m_FrameHandler; }

		void Remove()
			{ this->m_pNext->SetCurrent(); }

		void __stdcall UnwindNestedFrames(EXCEPTION_RECORD * pExcept);

		void SetCurrent()
		{
			__asm 
			{
				mov eax,this
				mov dword ptr fs:[0],eax
			}
		}

		static ExceptionFrame * GetCurrent()
		{
			ExceptionFrame * pRet;
			__asm 
			{
				mov eax, dword ptr fs:[0];
				mov pRet, eax
			}
			return pRet;
		}
	/// @}

	/// Members
	private:
		ExceptionFrame * m_pNext;
		void * m_FrameHandler;

	// Not to be implemented
	private:
		ExceptionFrame(const ExceptionFrame &);
		ExceptionFrame & operator=(const ExceptionFrame &);
	};

} // namespace __Platform

#endif /* __PLATFORM_WIN32_EXCEPTION_FRAME_H */
