/*

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __PLATFORM_WIN32_MUTEX_H
#define __PLATFORM_WIN32_MUTEX_H

#include "win32.h"

namespace __Platform
{
	class Mutex
	{
	public:
		Mutex()
		{
#ifdef _MT
			InitializeCriticalSection(&m_cs);
#endif
		}
		~Mutex()
		{
#ifdef _MT
			DeleteCriticalSection(&m_cs);
#endif
		}

		void Lock()
		{
#ifdef _MT
			EnterCriticalSection(&m_cs);
#endif
		}

		void Unlock()
		{
#ifdef _MT
			LeaveCriticalSection(&m_cs);
#endif
		}

	// Members
	private:
#ifdef _MT
		CRITICAL_SECTION m_cs;
#endif

	// Not to be implemented
	private:
		Mutex(Mutex const &);
		Mutex &operator =(Mutex const &);
	};

	class Lock
	{
	public:
		Lock(Mutex & mutex) 
#ifdef _MT
			: m_Mutex(mutex)
#endif
		{
#ifdef _MT
			m_Mutex.Lock();
#else
			mutex;
#endif
		}
		~Lock()
		{
#ifdef _MT
			m_Mutex.Unlock();
#endif
		}
	private:
#ifdef _MT
		Mutex & m_Mutex;
#endif

	// Not to be implemented
	private:
		Lock(Lock const &);
		Lock &operator =(Lock const &);
	};

} // namespace __Platform

#define SYNCHRONIZE_ON(x) { __Platform::Lock __lock(x);
#define END_SYNCHRONIZE   }

#endif /* __PLATFORM_WIN32_MUTEX_H */
