/*

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __PLATFORM_WIN32_STRING_H
#define __PLATFORM_WIN32_STRING_H

#include "win32.h"

namespace __Platform
{

	extern "C"
	{
		/* strcat */
		inline
		char *  __CRUNTINY_CRTCALL StrCat(char * dest, char const * src)
		{
			return lstrcatA(dest, src);
		}

		#define __PLATFORM_HAS_STRCAT


		/* wcscat */
		inline
		wchar_t *  __CRUNTINY_CRTCALL WcsCat(wchar_t * dest, wchar_t const * src)
		{
			return lstrcatW(dest, src);
		}

		#define __PLATFORM_HAS_WCSCAT


		/* strcmp */
		inline
		int  __CRUNTINY_CRTCALL StrCmp(char const * string1, char const * string2)
		{
			return lstrcmpA(string1, string2);
		}

		#define __PLATFORM_HAS_STRCMP


		/* wcscmp */
		inline
		int  __CRUNTINY_CRTCALL WcsCmp(wchar_t const * string1, wchar_t const * string2)
		{
			return lstrcmpW(string1, string2);
		}

		#define __PLATFORM_HAS_WCSCMP


		/* strcpy */
		inline
		char *  __CRUNTINY_CRTCALL StrCpy(char * dest, char const * src)
		{
			return lstrcpyA(dest, src);
		}

		#define __PLATFORM_HAS_STRCPY


		/* wcscpy */
		inline
		wchar_t * __CRUNTINY_CRTCALL WcsCpy(wchar_t * dest, wchar_t const * src)
		{
			return lstrcpyW(dest, src);
		}

		#define __PLATFORM_HAS_WCSCPY


		/* strlen */
		inline
		size_t  __CRUNTINY_CRTCALL StrLen(char const * string)
		{
			return lstrlenA(string);
		}

		#define __PLATFORM_HAS_STRLEN


		/* wcslen */
		inline
		size_t  __CRUNTINY_CRTCALL WcsLen(wchar_t const * string)
		{
			return lstrlenW(string);
		}

		#define __PLATFORM_HAS_WCSLEN


		/* strncpy */
/* **** lstrcpyn() does not have the same semantics are strncpy(), so cannot be used in its place
		inline
		char *  __CRUNTINY_CRTCALL StrNCpy(char * dest, char const * src, size_t count)
		{
			return lstrcpynA(dest, src, (int)count);
		}

		#define __PLATFORM_HAS_STRNCPY
*/

		/* wcsncpy */
/* **** lstrcpyn() does not have the same semantics are wcsncpy(), so cannot be used in its place
		inline
		wchar_t * __CRUNTINY_CRTCALL WcsNCpy(wchar_t * dest, wchar_t const * src, size_t count)
		{
			return lstrcpynW(dest, src, (int)count);
		}

		#define __PLATFORM_HAS_WCSNCPY
*/


		/* stricmp */
		inline
		int  __CRUNTINY_CRTCALL StrICmp(char const * string1, char const * string2)
		{
			return lstrcmpiA(string1, string2);
		}

		#define __PLATFORM_HAS_STRICMP


		/* wcsicmp */
		inline
		int  __CRUNTINY_CRTCALL WcsICmp(wchar_t const * string1, wchar_t const * string2)
		{
			return lstrcmpiW(string1, string2);
		}

		#define __PLATFORM_HAS_WCSICMP

#if 1 /* Testing out here */
		#undef __PLATFORM_HAS_STRCAT
		#undef __PLATFORM_HAS_WCSCAT
		#undef __PLATFORM_HAS_STRCMP
		#undef __PLATFORM_HAS_WCSCMP
		#undef __PLATFORM_HAS_STRCPY
		#undef __PLATFORM_HAS_WCSCPY
		#undef __PLATFORM_HAS_STRLEN
		#undef __PLATFORM_HAS_WCSLEN
		#undef __PLATFORM_HAS_STRNCPY
		#undef __PLATFORM_HAS_WCSNCPY
//		#undef __PLATFORM_HAS_STRICMP
//		#undef __PLATFORM_HAS_WCSICMP
#endif /* 1 */
	}

} // namespace __Platform

#endif /* __PLATFORM_WIN32_STRING_H */
