/*

Copyright (c) 2003, CRunTiny.org
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the names "CRunTiny" and "CRunTiny.org" nor the names of its 
contributors must not be used to endorse or promote products derived from 
this software without prior written permission. For written permission, please 
contact cruntiny@cruntiny.org.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef _INC_STDLIB
#define _INC_STDLIB


#include "crtcore.h"

#ifdef  _MSC_VER
#pragma pack(push,8)
#endif  

#ifdef  __cplusplus
extern "C" {
#endif


/* Definition of the argument values for the exit() function */

#define EXIT_SUCCESS    0
#define EXIT_FAILURE    1



/* Data structure definitions for div and ldiv runtimes. */

#ifndef _DIV_T_DEFINED

typedef struct _div_t {
        int quot;
        int rem;
} div_t;

typedef struct _ldiv_t {
        long quot;
        long rem;
} ldiv_t;

#define _DIV_T_DEFINED
#endif


/* Maximum value that can be returned by the rand function. */

#define RAND_MAX 0x7fff


/* Minimum and maximum macros */

#define __max(a,b)  (((a) > (b)) ? (a) : (b))
#define __min(a,b)  (((a) < (b)) ? (a) : (b))

/*
 * Sizes for buffers used by the _makepath() and _splitpath() functions.
 * note that the sizes include space for 0-terminator
 */
#define _MAX_PATH   260 /* max. length of full pathname */
#define _MAX_DRIVE  3   /* max. length of drive component */
#define _MAX_DIR    256 /* max. length of path component */
#define _MAX_FNAME  256 /* max. length of file name component */
#define _MAX_EXT    256 /* max. length of extension component */


/* External variable declarations */

#if     defined(_MT) || defined(_DLL)
	__CRUNTINY_EXPORT int * __CRUNTINY_CRTCALL _errno(void);
	#define errno       (*_errno())
#else   /* ndef _MT && ndef _DLL */
	__CRUNTINY_EXPORT extern int errno;               /* XENIX style error number */
	__CRUNTINY_EXPORT extern unsigned long _doserrno; /* OS system error value */
#endif  /* _MT || _DLL */


#if     defined(_DLL) && defined(_M_IX86)

#define __argc      (*__p___argc())     /* count of cmd line args */
#define __argv      (*__p___argv())     /* pointer to table of cmd line args */
#define __wargv     (*__p___wargv())    /* pointer to table of wide cmd line args */
#define _environ    (*__p__environ())   /* pointer to environment table */
#ifdef  _POSIX_
extern char ** environ;                 /* pointer to environment table */
#else
#define _wenviron   (*__p__wenviron())  /* pointer to wide environment table */
#endif  /* _POSIX_ */
#define _pgmptr     (*__p__pgmptr())    /* points to the module (EXE) name */
#define _wpgmptr    (*__p__wpgmptr())   /* points to the module (EXE) wide name */

__CRUNTINY_EXPORT int *          __CRUNTINY_CRTCALL __p___argc(void);
__CRUNTINY_EXPORT char ***       __CRUNTINY_CRTCALL __p___argv(void);
__CRUNTINY_EXPORT wchar_t ***    __CRUNTINY_CRTCALL __p___wargv(void);
__CRUNTINY_EXPORT char ***       __CRUNTINY_CRTCALL __p__environ(void);
__CRUNTINY_EXPORT wchar_t ***    __CRUNTINY_CRTCALL __p__wenviron(void);
__CRUNTINY_EXPORT char **        __CRUNTINY_CRTCALL __p__pgmptr(void);
__CRUNTINY_EXPORT wchar_t **     __CRUNTINY_CRTCALL __p__wpgmptr(void);


#else

__CRUNTINY_EXPORT extern int __argc;          /* count of cmd line args */
__CRUNTINY_EXPORT extern char ** __argv;      /* pointer to table of cmd line args */
__CRUNTINY_EXPORT extern wchar_t ** __wargv;  /* pointer to table of wide cmd line args */

#ifdef  _POSIX_
extern char ** environ;             /* pointer to environment table */
#else
__CRUNTINY_EXPORT extern char ** _environ;    /* pointer to environment table */
__CRUNTINY_EXPORT extern wchar_t ** _wenviron;    /* pointer to wide environment table */
#endif  /* _POSIX_ */

__CRUNTINY_EXPORT extern char * _pgmptr;      /* points to the module (EXE) name */
__CRUNTINY_EXPORT extern wchar_t * _wpgmptr;  /* points to the module (EXE) wide name */

#endif


__CRUNTINY_EXPORT extern int _fmode;          /* default file translation mode */
__CRUNTINY_EXPORT extern int _fileinfo;       /* open file info mode (for spawn) */


/* Windows major/minor and O.S. version numbers */

__CRUNTINY_EXPORT extern unsigned int _osplatform;
__CRUNTINY_EXPORT extern unsigned int _osver;
__CRUNTINY_EXPORT extern unsigned int _winver;
__CRUNTINY_EXPORT extern unsigned int _winmajor;
__CRUNTINY_EXPORT extern unsigned int _winminor;


/* function prototypes */

__CRUNTINY_EXPORT 
__CRUNTINY_DECLSPEC_NORETURN
void   __CRUNTINY_CRTCALL abort(void);


__CRUNTINY_EXPORT
__CRUNTINY_DECLSPEC_NORETURN
void   __CRUNTINY_CRTCALL exit(int);

#if     defined(_M_MRX000)
__CRUNTINY_EXPORT int    __CRUNTINY_CRTCALL abs(int);
#else
        int    __CRUNTINY_CRTCALL abs(int);
#endif
        __int64    __CRUNTINY_CRTCALL _abs64(__int64);
        int    __CRUNTINY_CRTCALL atexit(void (__CRUNTINY_CRTCALL *)(void));
__CRUNTINY_EXPORT double __CRUNTINY_CRTCALL atof(char const *);
__CRUNTINY_EXPORT int    __CRUNTINY_CRTCALL atoi(char const *);
__CRUNTINY_EXPORT long   __CRUNTINY_CRTCALL atol(char const *);
__CRUNTINY_EXPORT void * __CRUNTINY_CRTCALL bsearch(void const *, void const *, size_t, size_t,
        int (__CRUNTINY_CRTCALL *)(void const *, void const *));
        unsigned short __CRUNTINY_CRTCALL _byteswap_ushort(unsigned short);
        unsigned long  __CRUNTINY_CRTCALL _byteswap_ulong (unsigned long);
        unsigned __int64 __CRUNTINY_CRTCALL _byteswap_uint64(unsigned __int64);
__CRUNTINY_EXPORT void * __CRUNTINY_CRTCALL calloc(size_t, size_t);
__CRUNTINY_EXPORT div_t  __CRUNTINY_CRTCALL div(int, int);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL free(void *);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL getenv(char const *);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL _itoa(int, char *, int);
#if     _INTEGRAL_MAX_BITS >= 64
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL _i64toa(__int64, char *, int);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL _ui64toa(unsigned __int64, char *, int);
__CRUNTINY_EXPORT __int64 __CRUNTINY_CRTCALL _atoi64(char const *);
__CRUNTINY_EXPORT __int64 __CRUNTINY_CRTCALL _strtoi64(char const *, char **, int);
__CRUNTINY_EXPORT unsigned __int64 __CRUNTINY_CRTCALL _strtoui64(char const *, char **, int);
#endif
#if     defined(_M_MRX000)
__CRUNTINY_EXPORT long __CRUNTINY_CRTCALL labs(long);
#else
        long __CRUNTINY_CRTCALL labs(long);
#endif
__CRUNTINY_EXPORT ldiv_t __CRUNTINY_CRTCALL ldiv(long, long);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL _ltoa(long, char *, int);
__CRUNTINY_EXPORT void * __CRUNTINY_CRTCALL malloc(size_t);
__CRUNTINY_EXPORT int    __CRUNTINY_CRTCALL mblen(char const *, size_t);
__CRUNTINY_EXPORT size_t __CRUNTINY_CRTCALL _mbstrlen(char const *s);
__CRUNTINY_EXPORT int    __CRUNTINY_CRTCALL mbtowc(wchar_t *, char const *, size_t);
__CRUNTINY_EXPORT size_t __CRUNTINY_CRTCALL mbstowcs(wchar_t *, char const *, size_t);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL qsort(void *, size_t, size_t, int (__CRUNTINY_CRTCALL *)
        (void const *, void const *));
__CRUNTINY_EXPORT int    __CRUNTINY_CRTCALL rand(void);
__CRUNTINY_EXPORT void * __CRUNTINY_CRTCALL realloc(void *, size_t);
__CRUNTINY_EXPORT int    __CRUNTINY_CRTCALL _set_error_mode(int);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL srand(unsigned int);
__CRUNTINY_EXPORT double __CRUNTINY_CRTCALL strtod(char const *, char **);
__CRUNTINY_EXPORT long   __CRUNTINY_CRTCALL strtol(char const *, char **, int);
__CRUNTINY_EXPORT unsigned long __CRUNTINY_CRTCALL strtoul(char const *, char **, int);
__CRUNTINY_EXPORT int    __CRUNTINY_CRTCALL system(char const *);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL _ultoa(unsigned long, char *, int);
__CRUNTINY_EXPORT int    __CRUNTINY_CRTCALL wctomb(char *, wchar_t);
__CRUNTINY_EXPORT size_t __CRUNTINY_CRTCALL wcstombs(char *, wchar_t const *, size_t);


#ifndef _WSTDLIB_DEFINED

/* wide function prototypes, also declared in wchar.h  */

__CRUNTINY_EXPORT wchar_t * __CRUNTINY_CRTCALL _itow (int, wchar_t *, int);
__CRUNTINY_EXPORT wchar_t * __CRUNTINY_CRTCALL _ltow (long, wchar_t *, int);
__CRUNTINY_EXPORT wchar_t * __CRUNTINY_CRTCALL _ultow (unsigned long, wchar_t *, int);
__CRUNTINY_EXPORT double __CRUNTINY_CRTCALL wcstod(wchar_t const *, wchar_t **);
__CRUNTINY_EXPORT long   __CRUNTINY_CRTCALL wcstol(wchar_t const *, wchar_t **, int);
__CRUNTINY_EXPORT unsigned long __CRUNTINY_CRTCALL wcstoul(wchar_t const *, wchar_t **, int);
__CRUNTINY_EXPORT wchar_t * __CRUNTINY_CRTCALL _wgetenv(wchar_t const *);
__CRUNTINY_EXPORT int    __CRUNTINY_CRTCALL _wsystem(wchar_t const *);
__CRUNTINY_EXPORT double __CRUNTINY_CRTCALL _wtof(wchar_t const *);
__CRUNTINY_EXPORT int __CRUNTINY_CRTCALL _wtoi(wchar_t const *);
__CRUNTINY_EXPORT long __CRUNTINY_CRTCALL _wtol(wchar_t const *);
#if     _INTEGRAL_MAX_BITS >= 64
__CRUNTINY_EXPORT wchar_t * __CRUNTINY_CRTCALL _i64tow(__int64, wchar_t *, int);
__CRUNTINY_EXPORT wchar_t * __CRUNTINY_CRTCALL _ui64tow(unsigned __int64, wchar_t *, int);
__CRUNTINY_EXPORT __int64   __CRUNTINY_CRTCALL _wtoi64(wchar_t const *);
__CRUNTINY_EXPORT __int64   __CRUNTINY_CRTCALL _wcstoi64(wchar_t const *, wchar_t **, int);
__CRUNTINY_EXPORT unsigned __int64  __CRUNTINY_CRTCALL _wcstoui64(wchar_t const *, wchar_t **, int);
#endif

#define _WSTDLIB_DEFINED
#endif


#ifndef _POSIX_

__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL _ecvt(double, int, int *, int *);
__CRUNTINY_EXPORT __CRUNTINY_DECLSPEC_NORETURN void  __CRUNTINY_CRTCALL _exit(int);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL _fcvt(double, int, int *, int *);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL _fullpath(char *, char const *, size_t);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL _gcvt(double, int, char *);
        unsigned long __CRUNTINY_CRTCALL _lrotl(unsigned long, int);
        unsigned long __CRUNTINY_CRTCALL _lrotr(unsigned long, int);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL _makepath(char *, char const *, char const *, char const *,
        char const *);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL perror(char const *);
__CRUNTINY_EXPORT int    __CRUNTINY_CRTCALL _putenv(char const *);
        unsigned int __CRUNTINY_CRTCALL _rotl(unsigned int, int);
        unsigned __int64 __CRUNTINY_CRTCALL _rotl64(unsigned __int64, int);
        unsigned int __CRUNTINY_CRTCALL _rotr(unsigned int, int);
        unsigned __int64 __CRUNTINY_CRTCALL _rotr64(unsigned __int64, int);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL _searchenv(char const *, char const *, char *);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL _splitpath(char const *, char *, char *, char *, char *);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL _swab(char *, char *, int);

#ifndef _WSTDLIBP_DEFINED

/* wide function prototypes, also declared in wchar.h  */

__CRUNTINY_EXPORT wchar_t * __CRUNTINY_CRTCALL _wfullpath(wchar_t *, wchar_t const *, size_t);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL _wmakepath(wchar_t *, wchar_t const *, wchar_t const *, wchar_t const *,
        wchar_t const *);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL _wperror(wchar_t const *);
__CRUNTINY_EXPORT int    __CRUNTINY_CRTCALL _wputenv(wchar_t const *);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL _wsearchenv(wchar_t const *, wchar_t const *, wchar_t *);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL _wsplitpath(wchar_t const *, wchar_t *, wchar_t *, wchar_t *, wchar_t *);

#define _WSTDLIBP_DEFINED
#endif

/* --------- The following functions are OBSOLETE --------- */
/* The Win32 API SetErrorMode, Beep and Sleep should be used instead. */
__CRUNTINY_EXPORT void __CRUNTINY_CRTCALL _seterrormode(int);
__CRUNTINY_EXPORT void __CRUNTINY_CRTCALL _beep(unsigned, unsigned);
__CRUNTINY_EXPORT void __CRUNTINY_CRTCALL _sleep(unsigned long);
/* --------- The preceding functions are OBSOLETE --------- */

#endif  /* _POSIX_ */


#if     !__STDC__
/* --------- The declarations below should not be in stdlib.h --------- */
/* --------- and will be removed in a future release. Include --------- */
/* --------- ctype.h to obtain these declarations.            --------- */
#ifndef tolower     /* tolower has been undefined - use function */
__CRUNTINY_EXPORT int __CRUNTINY_CRTCALL tolower(int);
#endif  /* tolower */
#ifndef toupper     /* toupper has been undefined - use function */
__CRUNTINY_EXPORT int __CRUNTINY_CRTCALL toupper(int);
#endif  /* toupper */
/* --------- The declarations above will be removed.          --------- */
#endif


#if     !__STDC__

#ifndef _POSIX_

/* Non-ANSI names for compatibility */

#ifndef __cplusplus
#define max(a,b)    (((a) > (b)) ? (a) : (b))
#define min(a,b)    (((a) < (b)) ? (a) : (b))
#endif

#define sys_errlist _sys_errlist
#define sys_nerr    _sys_nerr
#define environ     _environ

__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL ecvt(double, int, int *, int *);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL fcvt(double, int, int *, int *);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL gcvt(double, int, char *);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL itoa(int, char *, int);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL ltoa(long, char *, int);
__CRUNTINY_EXPORT int    __CRUNTINY_CRTCALL putenv(char const *);
__CRUNTINY_EXPORT void   __CRUNTINY_CRTCALL swab(char *, char *, int);
__CRUNTINY_EXPORT char * __CRUNTINY_CRTCALL ultoa(unsigned long, char *, int);

#endif  /* _POSIX_ */

#endif  /* __STDC__ */

#ifdef  __cplusplus
}

#endif

#ifdef  _MSC_VER
#pragma pack(pop)
#endif  /* _MSC_VER */

#endif  /* _INC_STDLIB */
