/* /////////////////////////////////////////////////////////////////////////////
 * File:        stdio.c
 *
 * Purpose:     
 *
 * Created      10th July 2003
 * Updated:     8th December 2003
 *
 * Copyright:   cruntiny.org, (c) 2003. All rights reserved.
 *
 * Home:        www.cruntiny.org
 *
 * ////////////////////////////////////////////////////////////////////////// */


#include "cstdiotiny_internal.h"
#include "cstdiotiny_string.h"
#include "cstdiotiny.h"
#include <stdlib.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Function declarations
 */

extern unsigned long __cdecl _exception_code(void);
extern void *        __cdecl _exception_info(void);
extern int           __cdecl _abnormal_termination(void);

#ifndef EXCEPTION_EXECUTE_HANDLER
# define EXCEPTION_EXECUTE_HANDLER			(1)
#endif /* EXCEPTION_EXECUTE_HANDLER */
#ifndef EXCEPTION_CONTINUE_SEARCH
# define EXCEPTION_CONTINUE_SEARCH			(0)
#endif /* EXCEPTION_CONTINUE_SEARCH */
#ifndef EXCEPTION_CONTINUE_EXECUTION
# define EXCEPTION_CONTINUE_EXECUTION		(-1)
#endif /* EXCEPTION_CONTINUE_EXECUTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

typedef _win32_bool_t (__CSTDIOTINY_WIN32STDCALL *PFnWrite)(void *, void const *, unsigned long , unsigned long *, void *);

struct _cst_FILE
{
	void		*hFile;		/* The Win32 file HANDLE */
	PFnWrite	pfnWriteA;	/* A function that writes ANSI characters */
	PFnWrite	pfnWriteW;	/* A function that writes Unicode characters */
	int			error;		/* The stream error */

//	int		mode;
};

/* ////////////////////////////////////////////////////////////////////////// */

static _win32_bool_t __CSTDIOTINY_WIN32STDCALL WriteFileCharsA(void *hFile, char const *lpBuffer, unsigned long nNumberOfCharsToWrite, unsigned long *lpNumberOfCharsWritten, void *_dummy)
{
	return WriteFile(hFile, lpBuffer, nNumberOfCharsToWrite, lpNumberOfCharsWritten, 0);
}

static _win32_bool_t __CSTDIOTINY_WIN32STDCALL WriteFileCharsW(void *hFile, wchar_t const *lpBuffer, unsigned long nNumberOfCharsToWrite, unsigned long *lpNumberOfCharsWritten, void *_dummy)
{
	return WriteFile(hFile, lpBuffer, nNumberOfCharsToWrite * sizeof(wchar_t), lpNumberOfCharsWritten, 0);
}


FILE *_cstdiotiny_get_std_FILE(int i)
{
	static FILE				s_files[3];
	static _win32_bool_t	s_bInitialised	=	0;

	if(!s_bInitialised)
	{
		s_files[0].hFile		=	GetStdHandle((unsigned long)-10);
		s_files[0].pfnWriteA	=	NULL;
		s_files[0].pfnWriteW	=	NULL;
		s_files[0].error		=	0;

		s_files[1].hFile		=	GetStdHandle((unsigned long)-11);
		s_files[1].pfnWriteA	=	WriteConsoleA;
		s_files[1].pfnWriteW	=	WriteConsoleW;
		s_files[1].error		=	0;

		s_files[2].hFile		=	GetStdHandle((unsigned long)-12);
		s_files[2].pfnWriteA	=	WriteConsoleA;
		s_files[2].pfnWriteW	=	WriteConsoleW;
		s_files[2].error		=	0;

		s_bInitialised = 1;
	}

	return s_files + i;
};

_win32_bool_t _stdiotiny_strout_n(FILE *stream, char const *s, size_t len)
{
	unsigned long	_written;

	/* Since this needs to be an atomic operation, we first determine whether
	 * there are any \r\n to handle.
	 */
	if(NULL == strpbrk(s, "\r\n"))
	{
		return stream->pfnWriteA(stream->hFile, s, (unsigned long)len, &_written, 0);
	}
	else
	{
		_bool_t	success;
		char	_buffer[2048];
		char	*buffer = (NUM_ELEMENTS(_buffer) < (len * 2)) ? (char*)malloc(len * 2) : _buffer;

		if(NULL == buffer)
		{
			success = 0;
		}
		else
		{
			char const	*begin	=	s;
			char const	*end	=	begin + len;
			char const	*base	=	begin;
			char		*out	=	buffer;

			for(; begin != end; )
			{
				/* The cases we handle are:
				 *
				 * '\n' on its own	=>	translated into "\r\n"
				 * '\r' on its own	=>	translated into "\r\n"
				 * "\r\n"			-	which is left as is
				 */

				switch(*begin)
				{
					case	'\n':
insert_crlf:
						*out++ = '\r';
						*out++ = '\n';
						base = ++begin;
						break;
					case	'\r':
						/* Only if this is followed by \n do we not process */
						if(	begin + 1 == end ||
							*(begin + 1) != '\n')
						{
							goto insert_crlf;
						}
						/* FALL THROUGH HERE */
						*out++ = *begin++; /* Treat both \r and the following \n as ok */
					default:
						*out++ = *begin++;
						break;
				}
			}

			success = stream->pfnWriteA(stream->hFile, buffer, (unsigned long)(out - buffer), &_written, 0);

			GetLastError();

			if(buffer != _buffer)
			{
				free(buffer);
			}
		}

		return success;
	}
}

_win32_bool_t _stdiotiny_wcsout_n(FILE *stream, wchar_t const *s, size_t len)
{
	unsigned long	_written;

	/* Since this needs to be an atomic operation, we first determine whether
	 * there are any \r\n to handle.
	 */
	if(NULL == wcspbrk(s, L"\r\n"))
	{
		return stream->pfnWriteW(stream->hFile, s, (unsigned long)len, &_written, 0);
	}
	else
	{
		_bool_t	success;
		wchar_t	_buffer[1024];
		wchar_t	*buffer = (NUM_ELEMENTS(_buffer) < (len * 2)) ? (wchar_t*)malloc(sizeof(wchar_t) * (len * 2)) : _buffer;

		if(NULL == buffer)
		{
			success = 0;
		}
		else
		{
			wchar_t const	*begin	=	s;
			wchar_t const	*end	=	begin + len;
			wchar_t const	*base	=	begin;
			wchar_t			*out	=	buffer;

			for(; begin != end; )
			{
				/* The cases we handle are:
				 *
				 * '\n' on its own	=>	translated into "\r\n"
				 * '\r' on its own	=>	translated into "\r\n"
				 * "\r\n"			-	which is left as is
				 */

				switch(*begin)
				{
					case	'\n':
insert_crlf:
						*out++ = '\r';
						*out++ = '\n';
						base = ++begin;
						break;
					case	'\r':
						/* Only if this is followed by \n do we not process */
						if(	begin + 1 == end ||
							*(begin + 1) != '\n')
						{
							goto insert_crlf;
						}
						/* FALL THROUGH HERE */
						*out++ = *begin++; /* Treat both \r and the following \n as ok */
					default:
						*out++ = *begin++;
						break;
				}
			}

			success = stream->pfnWriteW(stream->hFile, buffer, (unsigned long)(out - buffer), &_written, 0);

			if(buffer != _buffer)
			{
				free(buffer);
			}
		}

		return success;
	}
}

#if 0
_win32_bool_t _stdiotiny_strout(void *stream, char const *s)
{
	return _stdiotiny_strout_n(stream, s, lstrlenA(s));
}
#endif /* 0 */

/* ////////////////////////////////////////////////////////////////////////// */

#define	MAX_PAGES		100
#define	PAGE_SIZE		4096	// This needs to be dynamically determined

long VAlloc(void **ppv, int *pages, _byte_t **nextPage, _win32_dword_t code)
{
	if(code != 0xC0000005L)
	{
		return EXCEPTION_EXECUTE_HANDLER;
	}
	else if(*pages >= MAX_PAGES)
    {
		return EXCEPTION_EXECUTE_HANDLER;
    }
	else
	{
		void	*pvNew = VirtualAlloc(	*nextPage,	// next page to commit
										PAGE_SIZE,	// page size, in bytes
										0x1000,      // allocate a committed page
										0x04);		// read/write access

		if(NULL == pvNew)
		{
			return EXCEPTION_EXECUTE_HANDLER;
		}

		(*pages)++;
		*nextPage += PAGE_SIZE;

		return EXCEPTION_CONTINUE_EXECUTION;
	}
}

/* ////////////////////////////////////////////////////////////////////////// */

/* printf() / wprintf() */

int __CSTDIOTINY_CRTCALL printf(char const *fmt, ...)
{
	va_list	args;
	int		res;

	va_start(args, fmt);
	res = vprintf(fmt, args);
	va_end(args);

	return res;
}

int __CSTDIOTINY_CRTCALL wprintf(wchar_t const *fmt, ...)
{
	va_list	args;
	int		res;

	va_start(args, fmt);
	res = vwprintf(fmt, args);
	va_end(args);

	return res;
}

/* sprintf() / wsprintf() */

int __CSTDIOTINY_CRTCALL sprintf(char *buffer, char const *fmt, ...)
{
	va_list	args;
	int		res;

	va_start(args, fmt);
	res = vsprintf(buffer, fmt, args);
	va_end(args);

	return res;
}

int __CSTDIOTINY_CRTCALL wsprintf(wchar_t *buffer, wchar_t const *fmt, ...)
{
	va_list	args;
	int		res;

	va_start(args, fmt);
	res = vswprintf(buffer, fmt, args);
	va_end(args);

	return res;
}

/* vprintf() / vwprintf() */

int __CSTDIOTINY_CRTCALL vprintf(char const *fmt, va_list args)
{
	return vfprintf(stdout, fmt, args);
}

int __CSTDIOTINY_CRTCALL vwprintf(wchar_t const *fmt, va_list args)
{
	return vfwprintf(stdout, fmt, args);
}

/* vsprintf() / vswprintf() */

int __CSTDIOTINY_CRTCALL vsprintf(char *buffer, char const *fmt, va_list args)
{
	return wvsprintfA(buffer, fmt, args);
}

int __CSTDIOTINY_CRTCALL vswprintf(wchar_t *buffer, wchar_t const *fmt, va_list args)
{
	return wvsprintfW(buffer, fmt, args);
}

/* putchar() / putwchar() */

int __CSTDIOTINY_CRTCALL putchar(int ch)
{
	return _stdiotiny_strout_n(GetStdHandle((unsigned long)-11), (char const *)&ch, 1) ? ch : EOF;
}

wint_t __CSTDIOTINY_CRTCALL putwchar(wint_t ch)
{
	return _stdiotiny_wcsout_n(GetStdHandle((unsigned long)-11), (wchar_t const *)&ch, 1) ? ch : EOF;
}

/* puts() / putws() */

int __CSTDIOTINY_CRTCALL puts(char const *s)
{
	return printf("%s\n", s) ? 0 : EOF;
}

int __CSTDIOTINY_CRTCALL putws(wchar_t const *s)
{
	return wprintf(L"%s\n", s) ? 0 : WEOF;
}

/* ////////////////////////////////////////////////////////////////////////// */

void __CSTDIOTINY_CRTCALL clearerr(FILE *stream)
{
	stream->error = 0;
}

int __CSTDIOTINY_CRTCALL ferror(FILE *stream)
{
	return stream->error;
}

int __CSTDIOTINY_CRTCALL fflush(FILE *stream)
{
	return FlushFileBuffers(stream->hFile) ? 0 : EOF;
}

int __CSTDIOTINY_CRTCALL fclose(FILE *stream)
{
	int	iRet;

	if(	stdin == stream ||
		stdout == stream ||
		stderr == stream)
	{
		OutputDebugStringA("Standard streams should not be closed\n");

		iRet = EOF;
	}
	else
	{
		FlushFileBuffers(stream->hFile);

		CloseHandle(stream->hFile);

		free(stream);

		iRet = 0;
	}

	return iRet;
}

#if 0
FILE * __CSTDIOTINY_CRTCALL fopen(char const *filename, char const *mode)
{
	FILE	*stream	=	(FILE*)malloc(sizeof(FILE));

}
#endif /* 0 */

int __CSTDIOTINY_CRTCALL fprintf(FILE *stream, char const *fmt, ...)
{
	va_list	args;
	int		res;

	va_start(args, fmt);
	res = vfprintf(stream, fmt, args);
	va_end(args);

	return res;
}

int __CSTDIOTINY_CRTCALL fwprintf(FILE *stream, wchar_t const *fmt, ...)
{
	va_list	args;
	int		res;

	va_start(args, fmt);
	res = vfwprintf(stream, fmt, args);
	va_end(args);

	return res;
}

int __CSTDIOTINY_CRTCALL vfprintf(FILE *stream, char const *fmt, va_list args)
{
	char	*buffer		=	(char*)VirtualAlloc(NULL, 25 * PAGE_SIZE, 0x2000, 0x01);
	int		pages		=	0;
	_byte_t *nextPage	=	(_byte_t*)buffer;
	int		res;

	__try
	{
		res		=	wvsprintfA(buffer, fmt, args);

		res		=	_stdiotiny_strout_n(stream, buffer, res) ? res : 0;
	}
	__except(VAlloc((void**)&buffer, &pages, &nextPage, _exception_code()))
	{
		res = 0;
	}

	VirtualFree(buffer, 0, 0x8000);

	return res;
}

int __CSTDIOTINY_CRTCALL vfwprintf(FILE *stream, wchar_t const *fmt, va_list args)
{
	wchar_t	*buffer		=	(wchar_t*)VirtualAlloc(NULL, 25 * PAGE_SIZE, 0x2000, 0x01);
	int		pages		=	0;
	_byte_t *nextPage	=	(_byte_t*)buffer;
	int		res;

	__try
	{
		res		=	wvsprintfW(buffer, fmt, args);

		res		=	_stdiotiny_wcsout_n(stream, buffer, res) ? res : 0;
	}
	__except(VAlloc((void**)&buffer, &pages, &nextPage, _exception_code()))
	{
		res = 0;
	}

	VirtualFree(buffer, 0, 0x8000);

	return res;
}

/* ////////////////////////////////////////////////////////////////////////// */
