/* /////////////////////////////////////////////////////////////////////////////
 * File:        memset.c
 *
 * Purpose:     implementation for memset.
 *
 * Created:     28th June 1999
 * Updated:     17th July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright (C) 2002, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *                          software@synesis-group.com
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software, or
 *              passed on by an intermediate supplier, at the time of
 *              distribution. Specifically, if a document stipulating one or
 *              more binary components are NOT to be redistributed is obtained
 *              from the supplier at the time of distribution then the user must
 *              (a) not redistribute any material named in that document, and
 *              (b) must redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis
 *              Software are placed into the source-code, executable programs
 *              and any associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLBASE
 #include <MLBase.h>
#endif /* !_SYNSOFT_INCL_H_MLBASE */
#ifndef _SYNSOFT_INCL_H_MLALIGN
 #include <MLAlign.h>
#endif /* !_SYNSOFT_INCL_H_MLALIGN */
#ifndef _SYNSOFT_INCL_H_MEDBGTRC
 #include <MEDbgTrc.h>
#endif /* !_SYNSOFT_INCL_H_MEDBGTRC */

/*
 * void *memset(void *, Int, size_t);
 * Int   memcmp(const void *, const void *, size_t);
 * void *memcpy(void *, const void *, size_t);
 * void *memmove(void *, const void *, size_t);
 */

#if __SYNSOFT_DVS_OS_BITSIZE == 32
 #define	PTR_SIZE				4
 #define	PtrWidthType			UInt32
 #define	PtrWidthInit(y)			((((UInt32)y) << 24) | (((UInt32)y) << 16) | (((UInt32)y) << 8) | (((UInt32)y)))
#elif __SYNSOFT_DVS_OS_BITSIZE == 16
 #define	PTR_SIZE				2
 #define	PtrWidthType			UInt16
 #define	PtrWidthInit(y)			((((UInt32)y) << 8) | (((UInt32)y)))
#else
 #error Not implemented for operating-system bit-sizes other than 16 & 32
#endif /* __SYNSOFT_DVS_OS_BITSIZE */
typedef	_mP(PtrWidthType)			PPtrWidthType;

SYFNCDECL(void*) memset(void *pvDest, Int c, size_t count)
{
	/* The strategy is simple. 
	 * 
	 * (i) If count > 4 * PTR_SIZE, then find the start and end points of the 
	 *     block that are aligned to the ambient pointer width. Set them in a 
	 *     for loop. 
	 * (ii) Do all the remainder front and end parts with byte loops.
	 */

	const Byte	ySet	=	SyCastStatic(Byte, c);
	PByte		begin	=	SyCastStatic(PByte, pvDest);
	PByte		end		=	begin + count;

	if(count > 4 * PTR_SIZE)
	{
		const PtrWidthType	tSet	=	PtrWidthInit(ySet);
		PPtrWidthType		pBegin	=	(PPtrWidthType)begin;
		PPtrWidthType		pEnd	=	(PPtrWidthType)end;
		Size				d_begin;
		Size				d_end;

#ifndef _SYNSOFT_INCL_H_SWRN003U
 #include <SWrn003u.h>
#endif /* !_SYNSOFT_INCL_H_SWRN003U */
		SyAlignUpToVoid(pBegin);
		SyAlignDownToVoid(pEnd);
#ifndef _SYNSOFT_INCL_H_SWRN003O
 #include <SWrn003o.h>
#endif /* !_SYNSOFT_INCL_H_SWRN003O */

		d_begin =	(PByte)pBegin - begin;
		d_end	=	end - (PByte)pEnd;

		for(; pBegin < pEnd; ++pBegin)
		{
			*pBegin = tSet;
		}

		if(d_begin != 0)
		{
			memset(begin, c, d_begin);
		}

		if(d_end != 0)
		{
			memset(pEnd, c, d_end);
		}
	}
	else
	{
		/* Do it as a byte array. */
		for(; begin < end; ++begin)
		{
			*begin = ySet;
		}
	}

	return pvDest;
}

/* ////////////////////////////////////////////////////////////////////////// */
