/* /////////////////////////////////////////////////////////////////////////////
 * File:        strrchr.c
 *
 * Purpose:     implementation for str/wcsrchr.
 *
 * Created:     15th December 1999
 * Updated:     17th July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright (C) 2002, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *                          software@synesis-group.com
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software, or
 *              passed on by an intermediate supplier, at the time of
 *              distribution. Specifically, if a document stipulating one or
 *              more binary components are NOT to be redistributed is obtained
 *              from the supplier at the time of distribution then the user must
 *              (a) not redistribute any material named in that document, and
 *              (b) must redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis
 *              Software are placed into the source-code, executable programs
 *              and any associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLBASE
 #include <MLBase.h>
#endif /* !_SYNSOFT_INCL_H_MLBASE */
#ifndef _SYNSOFT_INCL_H_MEDBGTRC
 #include <MEDbgTrc.h>
#endif /* !_SYNSOFT_INCL_H_MEDBGTRC */

#if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC
 #define __SYNSOFT_STRRCHR_NO_RETURN_TYPE_OVERLOAD
#endif /* __SYNSOFT_VAL_COMPILER_GNUC */

#ifndef _SYNSOFT_XINCL_C_STRRCHR

/* ANSI */
#undef	_SYNSOFT_XINCL_C_STRRCHR
#undef	XChar
#undef	XCharArg2
#undef	xT
#undef	strrchrX

#define	_SYNSOFT_XINCL_C_STRRCHR	0
#define	XChar						char
#ifdef __SYNSOFT_STRCHR_NO_RETURN_TYPE_OVERLOAD
 #define XCharArg2					int
#else
 #define XCharArg2					char
#endif /* __SYNSOFT_STRCHR_NO_RETURN_TYPE_OVERLOAD */
#define	xT(x)						x
#define	strrchrX					strrchr

#include "strrchr.c"

/* Unicode */
#undef	_SYNSOFT_XINCL_C_STRRCHR
#undef	XChar
#undef	XCharArg2
#undef	xT
#undef	strrchrX

#define	_SYNSOFT_XINCL_C_STRRCHR	1
#define	XChar						wchar_t
#define	XCharArg2					wchar_t
#define	xT(x)						L##x
#define	strrchrX					wcsrchr

#include "strrchr.c"

#else /* The implementation. */

/* strrchr functions.
 */

#ifdef __SYNSOFT_STRRCHR_NO_RETURN_TYPE_OVERLOAD
SYFNCDECLREF(XChar) strrchrX(XChar const * string, XCharArg2 ch)
#else
SYFNCDECLREF(XChar const) strrchrX(XChar const *string, XCharArg2 ch)
#endif /* __SYNSOFT_STRRCHR_NO_RETURN_TYPE_OVERLOAD */
{
	/* Iterate along the string, setting the return value for each match, so 
	 * that the last one is refered to automatically.
	 */
	XChar const	*ret = NULL;

	for(; *string; ++string)
	{
		if(*string == ch)
		{
			ret = string;
		}
	}

	return SyCastConst(XChar*, ret);
}

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_XINCL_C_STRRCHR */

/* ////////////////////////////////////////////////////////////////////////// */
