/* /////////////////////////////////////////////////////////////////////////////
 * File:        strtoul.c
 *
 * Purpose:     implementation for str/wcstoul.
 *
 * Created:     20th October 2001
 * Updated:     17th July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright (C) 2002, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *                          software@synesis-group.com
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software, or
 *              passed on by an intermediate supplier, at the time of
 *              distribution. Specifically, if a document stipulating one or
 *              more binary components are NOT to be redistributed is obtained
 *              from the supplier at the time of distribution then the user must
 *              (a) not redistribute any material named in that document, and
 *              (b) must redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis
 *              Software are placed into the source-code, executable programs
 *              and any associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLBASE
 #include <MLBase.h>
#endif /* !_SYNSOFT_INCL_H_MLBASE */
#include <ctype.h>
#include <limits.h>
#include <errno.h>
/* #include <MEDbgTrc.h> */

#ifndef _SYNSOFT_XINCL_C_STRTOUL

/* ANSI */
#undef	_SYNSOFT_XINCL_C_STRTOUL
#undef	XChar
#undef	xT
#undef	isdigitX
#undef	isspaceX
#undef	strtoulX

#define	_SYNSOFT_XINCL_C_STRTOUL	0
#define	XChar					char
#define	xT(x)					x
#define	strtoulX				strtoul
#define	isdigitX				isdigit
#define	isspaceX				isspace

#include "strtoul.c"

/* Unicode */
#undef	_SYNSOFT_XINCL_C_STRTOUL
#undef	XChar
#undef	xT
#undef	isdigitX
#undef	isspaceX
#undef	strtoulX

#define	_SYNSOFT_XINCL_C_STRTOUL	1
#define	XChar					wchar_t
#define	xT(x)					L##x
#define	strtoulX				wcstoul
#define	isdigitX				iswdigit
#define	isspaceX				iswspace

#include "strtoul.c"

#else /* The implementation. */

/* strtoul functions.
 */

#ifndef _SYNSOFT_INCL_H_SWRN005U
 #include <SWrn005u.h>
#endif /* !_SYNSOFT_INCL_H_SWRN005U */

#if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND
 #pragma warn -8004	/* Suppresses: "'xyz' is assigned a value that is never used in function 'ABC'" */
#endif /* __SYNSOFT_DVS_COMPILER */

#ifdef __SYNSOFT_DBS_IMPLEMENTATION_PARAMETERS_CONST_SUPPORT
SYFNCDECL(unsigned long) strtoulX(XChar const *nptr, XChar const **endptr, int base)
#else
SYFNCDECL(unsigned long) strtoulX(XChar const *nptr, XChar **endptr, int base)
#endif /* __SYNSOFT_DBS_DEBUG */
{
	unsigned long	ret			=	0;
	Bool			bNegative	=	false;
	Bool			bValid		=	true;

	for(; *nptr != xT('\0'); )
	{
		if(isdigitX(*nptr))
		{
			break;
		}
		else if(isspaceX(*nptr))
		{
			++nptr;
		}
		else
		{
			switch(*nptr)
			{
				default:
					bValid = false;
					break;
				case	xT('-'):
					bNegative = true;
					++nptr;
					break;
				case	xT('+'):
					bNegative = false;
					++nptr;
					break;
			}

			break;
		}
	}

	if(nptr[0] == xT('\0'))
	{
		bValid = false;
	}

	if(bValid)
	{
		/* Validate the base. */
		if(base == 0)
		{
			if(nptr[0] == xT('0'))
			{
				if(nptr[1] == xT('\0'))
				{
					bValid = false;
				}
				else
				{
					if(	nptr[1] == xT('x') ||
						nptr[1] == xT('X'))
					{
						++nptr;
						++nptr;

						base = 16;
					}
					else
					{
						++nptr;

						base = 8;
					}
				}
			}
			else
			{
				base = 10;
			}
		}

		if(bValid)
		{
			for(; *nptr && *nptr == xT('0'); ++nptr)
			{}

			/* Determine the starting figure. */
			switch(*nptr)
			{
				default:
				case	mT('0'):
					bValid = false;
					break;
				case	mT('1'):	ret	=	1;	break;
				case	mT('2'):	ret	=	2;	break;
				case	mT('3'):	ret	=	3;	break;
				case	mT('4'):	ret	=	4;	break;
				case	mT('5'):	ret	=	5;	break;
				case	mT('6'):	ret	=	6;	break;
				case	mT('7'):	ret	=	7;	break;
				case	mT('8'):	ret	=	8;	break;
				case	mT('9'):	ret	=	9;	break;
				case	mT('a'):	ret =	10;	break;
				case	mT('A'):	ret =	10;	break;
				case	mT('b'):	ret =	11;	break;
				case	mT('B'):	ret =	11;	break;
				case	mT('c'):	ret =	12;	break;
				case	mT('C'):	ret =	12;	break;
				case	mT('d'):	ret =	13;	break;
				case	mT('D'):	ret =	13;	break;
				case	mT('e'):	ret =	14;	break;
				case	mT('E'):	ret =	14;	break;
				case	mT('f'):	ret =	15;	break;
				case	mT('F'):	ret =	15;	break;
				case	mT('g'):	ret =	16;	break;
				case	mT('G'):	ret =	16;	break;
				case	mT('h'):	ret =	17;	break;
				case	mT('H'):	ret =	17;	break;
				case	mT('i'):	ret =	18;	break;
				case	mT('I'):	ret =	18;	break;
				case	mT('j'):	ret =	19;	break;
				case	mT('J'):	ret =	19;	break;
				case	mT('k'):	ret =	20;	break;
				case	mT('K'):	ret =	20;	break;
				case	mT('l'):	ret =	21;	break;
				case	mT('L'):	ret =	21;	break;
				case	mT('m'):	ret =	22;	break;
				case	mT('M'):	ret =	22;	break;
				case	mT('n'):	ret =	23;	break;
				case	mT('N'):	ret =	23;	break;
				case	mT('o'):	ret =	24;	break;
				case	mT('O'):	ret =	24;	break;
				case	mT('p'):	ret =	25;	break;
				case	mT('P'):	ret =	25;	break;
				case	mT('q'):	ret =	26;	break;
				case	mT('Q'):	ret =	26;	break;
				case	mT('r'):	ret =	27;	break;
				case	mT('R'):	ret =	27;	break;
				case	mT('s'):	ret =	28;	break;
				case	mT('S'):	ret =	28;	break;
				case	mT('t'):	ret =	29;	break;
				case	mT('T'):	ret =	29;	break;
				case	mT('u'):	ret =	30;	break;
				case	mT('U'):	ret =	30;	break;
				case	mT('v'):	ret =	31;	break;
				case	mT('V'):	ret =	31;	break;
				case	mT('w'):	ret =	32;	break;
				case	mT('W'):	ret =	32;	break;
				case	mT('x'):	ret =	33;	break;
				case	mT('X'):	ret =	33;	break;
				case	mT('y'):	ret =	34;	break;
				case	mT('Y'):	ret =	34;	break;
				case	mT('z'):	ret =	35;	break;
				case	mT('Z'):	ret =	35;	break;
			}

			if(	/* ret < 0 ||
				 */!(ret < (unsigned long)base))
			{
				bValid = false;
			}

			if(bValid)
			{
				++nptr;

				/* Now iterate through the string, multiplying. */
				for(; *nptr; ++nptr)
				{
					unsigned long	digit;

					switch(*nptr)
					{
						default:
							digit = ULONG_MAX;
							break;
						case	mT('0'):	digit = 0;		break;
						case	mT('1'):	digit = 1;		break;
						case	mT('2'):	digit = 2;		break;
						case	mT('3'):	digit = 3;		break;
						case	mT('4'):	digit = 4;		break;
						case	mT('5'):	digit = 5;		break;
						case	mT('6'):	digit = 6;		break;
						case	mT('7'):	digit = 7;		break;
						case	mT('8'):	digit = 8;		break;
						case	mT('9'):	digit = 9;		break;
						case	mT('a'):	digit = 10;		break;
						case	mT('A'):	digit = 10;		break;
						case	mT('b'):	digit = 11;		break;
						case	mT('B'):	digit = 11;		break;
						case	mT('c'):	digit = 12;		break;
						case	mT('C'):	digit = 12;		break;
						case	mT('d'):	digit = 13;		break;
						case	mT('D'):	digit = 13;		break;
						case	mT('e'):	digit = 14;		break;
						case	mT('E'):	digit = 14;		break;
						case	mT('f'):	digit = 15;		break;
						case	mT('F'):	digit = 15;		break;
						case	mT('g'):	digit = 16;		break;
						case	mT('G'):	digit = 16;		break;
						case	mT('h'):	digit = 17;		break;
						case	mT('H'):	digit = 17;		break;
						case	mT('i'):	digit = 18;		break;
						case	mT('I'):	digit = 18;		break;
						case	mT('j'):	digit = 19;		break;
						case	mT('J'):	digit = 19;		break;
						case	mT('k'):	digit = 20;		break;
						case	mT('K'):	digit = 20;		break;
						case	mT('l'):	digit = 21;		break;
						case	mT('L'):	digit = 21;		break;
						case	mT('m'):	digit = 22;		break;
						case	mT('M'):	digit = 22;		break;
						case	mT('n'):	digit = 23;		break;
						case	mT('N'):	digit = 23;		break;
						case	mT('o'):	digit = 24;		break;
						case	mT('O'):	digit = 24;		break;
						case	mT('p'):	digit = 25;		break;
						case	mT('P'):	digit = 25;		break;
						case	mT('q'):	digit = 26;		break;
						case	mT('Q'):	digit = 26;		break;
						case	mT('r'):	digit = 27;		break;
						case	mT('R'):	digit = 27;		break;
						case	mT('s'):	digit = 28;		break;
						case	mT('S'):	digit = 28;		break;
						case	mT('t'):	digit = 29;		break;
						case	mT('T'):	digit = 29;		break;
						case	mT('u'):	digit = 30;		break;
						case	mT('U'):	digit = 30;		break;
						case	mT('v'):	digit = 31;		break;
						case	mT('V'):	digit = 31;		break;
						case	mT('w'):	digit = 32;		break;
						case	mT('W'):	digit = 32;		break;
						case	mT('x'):	digit = 33;		break;
						case	mT('X'):	digit = 33;		break;
						case	mT('y'):	digit = 34;		break;
						case	mT('Y'):	digit = 34;		break;
						case	mT('z'):	digit = 35;		break;
						case	mT('Z'):	digit = 35;		break;
					}

					if(	digit == ULONG_MAX ||
						!(digit < (unsigned long)base))
					{
						bValid = false;

						break;
					}
					else
					{
						unsigned long	l	=	ULONG_MAX / base;
						unsigned long	m	=	ULONG_MAX % base;

						if(	l < ret ||
							(	l == ret &&
								m < digit))
						{
							errno = ERANGE;

							ret = ULONG_MAX;
							++nptr;

							break;
						}
						else
						{
							ret = ret * base + digit;
						}

						if(!bValid)
						{
							break;
						}
					}
				}
			}
		}
	}

	if(endptr != NULL)
	{
		*endptr = SyCastConst(XChar*, nptr);
	}

	if(bNegative)
	{
		ret = (unsigned long)-(long)ret;
	}

	return ret;
}

#if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND
 #pragma warn .8004	/* Reinstates: "'xyz' is assigned a value that is never used in function 'ABC'" */
#endif /* __SYNSOFT_DVS_COMPILER */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _SYNSOFT_INCL_H_SWRN005O
 #include <SWrn005o.h>
#endif /* !_SYNSOFT_INCL_H_SWRN005O */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_XINCL_C_STRTOUL */

/* ////////////////////////////////////////////////////////////////////////// */
