/* /////////////////////////////////////////////////////////////////////////////
 * File:        cstdiotiny.h
 *
 * Purpose:     Main header for CStdioTiny
 *
 * Created      9th July 2003
 * Updated:     8th December 2003
 *
 * Copyright:   cruntiny.org, (c) 2003. All rights reserved.
 *
 * Home:        www.cruntiny.org
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef __CSTDIOTINY_INCL_H_CSTDIOTINY
#define __CSTDIOTINY_INCL_H_CSTDIOTINY

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <crtcore.h>	/* Dependent on CRunTiny */
#include <stdarg.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Version information
 */

#define	_CSTDIOTINY_VER_MAJOR	0
#define	_CSTDIOTINY_VER_MINOR	3

#ifndef WIN32
# error This version of CStdioTiny is only valid for Win32 compilation
#endif /* WIN32 */

/* /////////////////////////////////////////////////////////////////////////////
 * function specifications
 */

/* Linkage class
 */

#define	__CSTDIOTINY_MOD_LINKAGE_C		__CRUNTINY_MOD_LINKAGE_C

/* Storage class
 */

/* Calling convention
 */

#define __CSTDIOTINY_CRTCALL			__CRUNTINY_CRTCALL
#define __CSTDIOTINY_WIN32STDCALL		__stdcall
#define __CSTDIOTINY_WIN32CRTCALL		__cdecl

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

/* size_t */
#ifndef _SIZE_T_DEFINED
  typedef	unsigned int				size_t;
#  define _SIZE_T_DEFINED
#endif /* _SIZE_T_DEFINED */

/* wchar_t */
#ifndef _WCHAR_T_DEFINED
  typedef unsigned short				wchar_t;
#  define _WCHAR_T_DEFINED
#endif /* _WCHAR_T_DEFINED */

/* wint_t */
#ifndef _WCTYPE_T_DEFINED
  typedef wchar_t						wint_t;
  typedef wchar_t						wctype_t;
#  define _WCTYPE_T_DEFINED
#endif /* !_WCTYPE_T_DEFINED */

struct _cst_FILE;

typedef struct _cst_FILE				FILE;

typedef __int64							fpos_t;


/* /////////////////////////////////////////////////////////////////////////////
 * Constants
 */

#ifndef NULL
#  ifdef __cplusplus
#    define NULL						0
#  else
#    define NULL						((void *)0)
#  endif /* __cplusplus */
#endif /* !NULL */

#ifndef EOF
#  define EOF							(-1)
#endif /* !EOF */

#ifndef WEOF
#  define WEOF							((wint_t)(0xFFFF))
#endif /* !WEOF */

#ifdef __cplusplus
  extern "C" FILE *_cstdiotiny_get_std_FILE(int i);
#else
  extern FILE *_cstdiotiny_get_std_FILE(int i);
#endif /* __cplusplus */

#define stdin							(_cstdiotiny_get_std_FILE(0))
#define stdout							(_cstdiotiny_get_std_FILE(1))
#define stderr							(_cstdiotiny_get_std_FILE(2))

/* /////////////////////////////////////////////////////////////////////////////
 * Macros
 */

#ifndef NUM_ELEMENTS
# define NUM_ELEMENTS(_x)				(sizeof(_x) / sizeof(0[_x]))
#endif /* !NUM_ELEMENTS */

/* /////////////////////////////////////////////////////////////////////////////
 * CStdioTiny functions
 */

extern void cstdiotiny_init();
extern void cstdiotiny_uninit();

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

/* Implemented */
/*
void *	__CSTDIOTINY_CRTCALL memccpy(void *, void const *, int, unsigned int);
int		__CSTDIOTINY_CRTCALL memicmp(void const *, void const *, unsigned int);
*/


__CRUNTINY_EXPORT void __CSTDIOTINY_CRTCALL clearerr(FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fclose(FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL feof(FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL ferror(FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fflush(FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fgetc(FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fgetpos(FILE *, fpos_t *);
__CRUNTINY_EXPORT char * __CSTDIOTINY_CRTCALL fgets(char *, int, FILE *);
__CRUNTINY_EXPORT FILE * __CSTDIOTINY_CRTCALL fopen(char const *, char const *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fprintf(FILE *, char const *, ...);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fputc(int, FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fputs(char const *, FILE *);
__CRUNTINY_EXPORT size_t __CSTDIOTINY_CRTCALL fread(void *, size_t, size_t, FILE *);
__CRUNTINY_EXPORT FILE * __CSTDIOTINY_CRTCALL freopen(char const *, char const *, FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fscanf(FILE *, char const *, ...);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fsetpos(FILE *, const fpos_t *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fseek(FILE *, long, int);
__CRUNTINY_EXPORT long __CSTDIOTINY_CRTCALL ftell(FILE *);
__CRUNTINY_EXPORT size_t __CSTDIOTINY_CRTCALL fwrite(const void *, size_t, size_t, FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL getc(FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL getchar(void);
__CRUNTINY_EXPORT char * __CSTDIOTINY_CRTCALL gets(char *);
__CRUNTINY_EXPORT void __CSTDIOTINY_CRTCALL perror(char const *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL printf(char const *, ...);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL putc(int, FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL putchar(int);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL puts(char const *s);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL remove(char const *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL rename(char const *, char const *);
__CRUNTINY_EXPORT void __CSTDIOTINY_CRTCALL rewind(FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL scanf(char const *, ...);
__CRUNTINY_EXPORT void __CSTDIOTINY_CRTCALL setbuf(FILE *, char *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL setvbuf(FILE *, char *, int, size_t);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL sprintf(char *, char const *, ...);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL sscanf(char const *, char const *, ...);
__CRUNTINY_EXPORT FILE * __CSTDIOTINY_CRTCALL tmpfile(void);
__CRUNTINY_EXPORT char * __CSTDIOTINY_CRTCALL tmpnam(char *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL ungetc(int, FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL vfprintf(FILE *, char const *, va_list);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL vprintf(char const *, va_list);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL vsprintf(char *, char const *, va_list);

__CRUNTINY_EXPORT wint_t __CSTDIOTINY_CRTCALL fgetwc(FILE *);
__CRUNTINY_EXPORT wint_t __CSTDIOTINY_CRTCALL fputwc(wint_t, FILE *);
__CRUNTINY_EXPORT wint_t __CSTDIOTINY_CRTCALL getwc(FILE *);
__CRUNTINY_EXPORT wint_t __CSTDIOTINY_CRTCALL getwchar(void);
__CRUNTINY_EXPORT wint_t __CSTDIOTINY_CRTCALL putwc(wint_t, FILE *);
__CRUNTINY_EXPORT wint_t __CSTDIOTINY_CRTCALL putwchar(wint_t);
__CRUNTINY_EXPORT wint_t __CSTDIOTINY_CRTCALL ungetwc(wint_t, FILE *);
__CRUNTINY_EXPORT wchar_t * __CSTDIOTINY_CRTCALL fgetws(wchar_t *, int, FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fputws(wchar_t const *, FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fwprintf(FILE *, wchar_t const *, ...);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL wprintf(wchar_t const *, ...);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL swprintf(wchar_t *, wchar_t const *, ...);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL vfwprintf(FILE *, wchar_t const *, va_list);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL vwprintf(wchar_t const *, va_list);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL vswprintf(wchar_t *, wchar_t const *, va_list);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fwscanf(FILE *, wchar_t const *, ...);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL swscanf(wchar_t const *, wchar_t const *, ...);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL wscanf(wchar_t const *, ...);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fcloseall(void);
__CRUNTINY_EXPORT FILE * __CSTDIOTINY_CRTCALL fdopen(int, char const *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fgetchar(void);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fileno(FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL flushall(void);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL fputchar(int);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL getw(FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL putw(int, FILE *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL		rmtmp(void);
__CRUNTINY_EXPORT char * __CSTDIOTINY_CRTCALL	tempnam(char const *, char const *);
__CRUNTINY_EXPORT int __CSTDIOTINY_CRTCALL		unlink(char const *);

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* __CSTDIOTINY_INCL_H_CSTDIOTINY */

/* ////////////////////////////////////////////////////////////////////////// */
